﻿using System;
using System.IO;

namespace IntelliSite.CommonUtil
{
	[Message]
	public class ConnectMessage : MessageBase
	{
		public const short MessageType = 1;

		public string ClientName { get; private set; }
		public byte ClientType { get; private set; }
		public string Version { get; private set; }

		public ConnectMessage( string clientName, byte clientType = 0, string version=default( string ) )
		{
			Type = MessageType;
			ClientName = clientName;
			ClientType = clientType;
			Version = version ?? String.Empty;

			using( MemoryStream memStream = new MemoryStream() )
			using( BinaryWriter writer = new BinaryWriter( memStream ) )
			{
				writer.Write( clientName );
				writer.Write( clientType );
				writer.Write( Version );
				Data = memStream.ToArray();
			}
		}

		public ConnectMessage( MessageBase message )
			: base( message )
		{
			using( MemoryStream memStream = new MemoryStream( Data ) )
			using( BinaryReader reader = new BinaryReader( memStream ) )
			{
				ClientName = reader.ReadString();
				ClientType = reader.ReadByte();
				Version = reader.ReadString();
			}
		}

		public override string ToString()
		{
			return "Connect: " + ClientName + " Type: " + ClientType + " Version: " + Version;
		}
	}
}
