﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using IntelliSite.CommonUtil.Extensions;

namespace IntelliSite.CommonUtil
{
	public abstract class DisposableBase : IDisposable
	{
		private bool _disposed = false;

		protected DisposableBase()
		{
		}

		public bool IsDisposed { get { return _disposed; } }

		public event EventHandler Disposing;
		public event EventHandler Disposed;

		public void Dispose()
		{
			if( _disposed )
				return;
			FireDisposing( EventArgs.Empty );
			Dispose( disposing: true );
			_disposed = true;
			FireDisposed( EventArgs.Empty );
		}

		protected abstract void Dispose( bool disposing );

		protected void FireDisposed( EventArgs e )
		{
			Disposed.FireEvent( this, e );
		}

		protected void FireDisposing( EventArgs e )
		{
			Disposing.FireEvent( this, e );
		}

		protected virtual void GuardDisposed()
		{
			if( _disposed )
				throw new ObjectDisposedException( "Cannot access disposed object." );
		}
	}
}
