﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;

namespace IntelliSite.CommonUtil.Extensions
{
	public static class AssemblyExtensions
	{
		public static IEnumerable<KeyValuePair<TAttributeKey, Type>> FindTypesWithAttribute<TAttributeKey>( this Assembly assembly, bool? inherit = null ) where TAttributeKey : Attribute
		{
			var attributeType = typeof( TAttributeKey );

			bool lookWithInherit;
			if( inherit.HasValue )
			{
				lookWithInherit = inherit.Value;
			}
			else
			{
				lookWithInherit = true;

				var attrs = attributeType.GetCustomAttributes( typeof( AttributeUsageAttribute ), true ).OfType<AttributeUsageAttribute>();
				foreach( var attr in attrs )
				{
					lookWithInherit = attr.Inherited;
				}
			}

			foreach( var type in assembly.GetTypes() )
			{
				var attrs = type.GetCustomAttributes( attributeType, lookWithInherit ).OfType<TAttributeKey>();
				foreach( var attr in attrs )
				{
					var pair = new KeyValuePair<TAttributeKey, Type>( attr, type );
					yield return pair;
				}
			}
		}
	}
}
