﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Win32;

namespace IntelliSite.CommonUtil.Extensions
{
	public static class RegistryKeyExtensions
	{
		public static T GetValue<T>( this RegistryKey key, string name ) where T : class
		{
			if( key != null )
			{
				object rawValue = key.GetValue( name );
				return (T)rawValue;
			}
			return null;
		}

		public static T GetValue<T>( this RegistryKey key, string name, T defaultValue )
		{
			if( key != null )
			{
				object rawValue = key.GetValue( name );
				if( rawValue != null )
					return (T)rawValue;
			}
			return defaultValue;
		}

		public static bool GetValueAsBool( this RegistryKey key, string name, bool defaultValue )
		{
			if( key != null )
			{
				object rawValue = key.GetValue( name );
				if( rawValue != null )
					return (int)rawValue != 0;
			}
			return defaultValue;
		}

		public static UInt16 GetValueAsUInt16( this RegistryKey key, string name, UInt16 defaultValue )
		{
			if( key != null )
			{
				object rawValue = key.GetValue( name );
				if( rawValue != null )
					return (UInt16)(int)rawValue;
			}
			return defaultValue;
		}

		public static Version GetValueAsVersion( this RegistryKey key, string name )
		{
			return GetValueAsVersion( key, name, defaultValue: null );
		}

		public static Version GetValueAsVersion( this RegistryKey key, string name, Version defaultValue )
		{
			if( key != null )
			{
				object rawValue = key.GetValue( name );
				if( rawValue != null )
					return Version.Parse( (string)rawValue );
			}
			return defaultValue;
		}

		public static void SetEncodedValue( this RegistryKey key, string name, bool value )
		{
			int encodedValue = value ? 1 : 0;
			key.SetValue( name, encodedValue, RegistryValueKind.DWord );
		}

		public static void SetEncodedValue( this RegistryKey key, string name, UInt16 value )
		{
			int encodedValue = value;
			key.SetValue( name, encodedValue, RegistryValueKind.DWord );
		}

		public static void SetEncodedValue( this RegistryKey key, string name, Version value )
		{
			string encodedValue = value != null ? value.ToString() : null;
			key.SetValue( name, encodedValue, RegistryValueKind.String );
		}
	}
}
