﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Runtime.InteropServices;
using System.Runtime.InteropServices.ComTypes;
using System.Security;
using System.Text;

namespace IntelliSite.CommonUtil.FusionApi
{
	#region Native functions

	internal static class Constants
	{
		public const string FUSION_REFCOUNT_UNINSTALL_SUBKEY_GUID = "{8cedc215-ac4b-488b-93c0-a50a49cb2fb8}";
		public const string FUSION_REFCOUNT_FILEPATH_GUID = "{b02f9d65-fb77-4f7a-afa5-b391309f11c9}";
		public const string FUSION_REFCOUNT_OPAQUE_STRING_GUID = "{2ec93463-b0c3-45e1-8364-327e96aea856}";
		public const string FUSION_REFCOUNT_MSI_GUID = "{25df0fc1-7f97-4070-add7-4b13bbfd7cb8}"; // this GUID cannot be used for installing into GAC.
		public const string FUSION_REFCOUNT_OSINSTALL_GUID = "{d16d444c-56d8-11d5-882d-0080c847b195}";
	}

	internal static class Native
	{
		[DllImport( "fusion.dll", EntryPoint = "CreateAssemblyConfigCookie", CharSet = CharSet.Unicode, PreserveSig = false )]
		[return: MarshalAs( UnmanagedType.Error )]
		internal static extern void CreateAssemblyConfigCookie(
			[MarshalAs( UnmanagedType.LPWStr )] string wzConfigFilePath,
			out IntPtr ppAssemblyConfigOut // struct AssemblyConfig **
			);

		[DllImport( "fusion.dll", EntryPoint = "DestroyAssemblyConfigCookie", CharSet = CharSet.Unicode, PreserveSig = false )]
		[return: MarshalAs( UnmanagedType.Error )]
		internal static extern void DestroyAssemblyConfigCookie(
			IntPtr pAssemblyConfig // struct AssemblyConfig *
			);

		[DllImport( "fusion.dll", EntryPoint = "CompareAssemblyIdentity", PreserveSig = false )]
		[return: MarshalAs( UnmanagedType.Error )]
		internal static extern void CompareAssemblyIdentity(
			[MarshalAs( UnmanagedType.LPWStr )] string pwzAssemblyIdentity1,
			[MarshalAs( UnmanagedType.Bool )] bool fUnified1,
			[MarshalAs( UnmanagedType.LPWStr )] string pwzAssemblyIdentity2,
			[MarshalAs( UnmanagedType.Bool )] bool fUnified2,
			[MarshalAs( UnmanagedType.Bool )] out bool pfEquivalent,
			out AssemblyComparisonResult pResult
			);

		[DllImport( "fusion.dll", EntryPoint = "CompareAssemblyIdentityWithConfig", PreserveSig = false )]
		[return: MarshalAs( UnmanagedType.Error )]
		internal static extern void CompareAssemblyIdentityWithConfig(
			[MarshalAs( UnmanagedType.LPWStr )] string pwzAssemblyIdentity1,
			[MarshalAs( UnmanagedType.Bool )] bool fUnified1,
			[MarshalAs( UnmanagedType.LPWStr )] string pwzAssemblyIdentity2,
			[MarshalAs( UnmanagedType.Bool )] bool fUnified2,
			IntPtr pAssemblyConfig, // struct AssemblyConfig *
			[MarshalAs( UnmanagedType.Bool )] out bool pfEquivalent,
			out AssemblyComparisonResult pResult
			);

		[DllImport( "fusion.dll", EntryPoint = "CreateInstallReferenceEnum", PreserveSig = false )]
		[return: MarshalAs( UnmanagedType.Error )]
		internal static extern void CreateInstallReferenceEnum(
			out IInstallReferenceEnum ppRefEnum,
			IAssemblyName pName,
			uint dwFlags,
			[Optional] IntPtr pvReserved
			);

		[DllImport( "fusion.dll", EntryPoint = "CreateAssemblyEnum", PreserveSig = false )]
		[return: MarshalAs( UnmanagedType.Error )]
		internal static extern void CreateAssemblyEnum(
			out IAssemblyEnum pEnum,
			[Optional] IntPtr pUnkReserved,
			IAssemblyName pName,
			[Optional] ASM_CACHE_FLAGS dwFlags,
			[Optional] IntPtr pvReserved
			);

		[DllImport( "fusion.dll", EntryPoint = "CreateAssemblyEnum", PreserveSig = false )]
		[return: MarshalAs( UnmanagedType.Error )]
		internal static extern void CreateAssemblyEnum(
			out IAssemblyEnum pEnum,
			[Optional] IntPtr pUnkReserved,
			[Optional] IntPtr pName,
			[Optional] ASM_CACHE_FLAGS dwFlags,
			[Optional] IntPtr pvReserved
			);

		[DllImport( "fusion.dll", EntryPoint = "CreateAssemblyNameObject", PreserveSig = false )]
		[return: MarshalAs( UnmanagedType.Error )]
		internal static extern void CreateAssemblyNameObject(
			out IAssemblyName ppAssemblyNameObj,
			[MarshalAs( UnmanagedType.LPWStr )] string szAssemblyName,
			[Optional] CREATE_ASM_NAME_OBJ_FLAGS dwFlags,
			[Optional] IntPtr pvReserved
			);

		[DllImport( "fusion.dll", EntryPoint = "CreateAssemblyCache", PreserveSig = false )]
		[return: MarshalAs( UnmanagedType.Error )]
		internal static extern void CreateAssemblyCache(
			out IAssemblyCache ppAsmCache,
			[Optional] uint dwReserved
			);

		[DllImport( "fusion.dll", EntryPoint = "GetCachePath", PreserveSig = false )]
		[return: MarshalAs( UnmanagedType.Error )]
		internal static extern void GetCachePath(
			ASM_CACHE_FLAGS dwCacheFlags,
			[MarshalAs( UnmanagedType.LPWStr )] out string pwzCachePath,
			out uint pcchPath
			);

		[DllImport( "fusion.dll", EntryPoint = "GetAssemblyIdentityFromFile", PreserveSig = false )]
		[return: MarshalAs( UnmanagedType.Error )]
		internal static extern void GetAssemblyIdentityFromFile(
			[MarshalAs( UnmanagedType.LPWStr )] string pwzFilePath,
			[In, Out] ref Guid riid,
			[MarshalAs( UnmanagedType.Interface )] out object ppIdentity
			);

		[DllImport( "fusion.dll", EntryPoint = "ClearDownloadCache", PreserveSig = false )]
		[return: MarshalAs( UnmanagedType.Error )]
		internal static extern void ClearDownloadCache(
			);

		[DllImport( "fusion.dll", EntryPoint = "SetMSIHandleForLogging", PreserveSig = false )]
		[return: MarshalAs( UnmanagedType.Error )]
		internal static extern void SetMSIHandleForLogging(
			uint hMSIHandle // MSIHANDLE
			);
	}

	#endregion

	#region COM interfaces

	[ComImport]
	[InterfaceType( ComInterfaceType.InterfaceIsIUnknown )]
	[Guid( "7c23ff90-33af-11d3-95da-00a024a85b51" )]
	internal interface IApplicationContext
	{
		void SetContextNameObject(
			IAssemblyName pName );

		void GetContextNameObject(
			out IAssemblyName ppName );

		void Set(
			[MarshalAs( UnmanagedType.LPWStr )] string szName,
			int pvValue,
			uint cbValue,
			uint dwFlags );

		void Get(
			[MarshalAs( UnmanagedType.LPWStr )] string szName,
			out int pvValue,
			ref uint pcbValue,
			uint dwFlags );

		void GetDynamicDirectory(
			out int wzDynamicDir,
			ref uint pdwSize );
	}

	[ComImport]
	[InterfaceType( ComInterfaceType.InterfaceIsIUnknown )]
	[Guid( "E707DCDE-D1CD-11D2-BAB9-00C04F8ECEAE" )]
	internal interface IAssemblyCache
	{
		void UninstallAssembly(
			uint dwFlags,
			[MarshalAs( UnmanagedType.LPWStr )] string pszAssemblyName,
			[In] ref FUSION_INSTALL_REFERENCE pRefData,
			[Optional] out uint pulDisposition );
		
		void QueryAssemblyInfo(
			[MarshalAs( UnmanagedType.U4 )] QueryAsmInfo_Flag dwFlags,
			[MarshalAs( UnmanagedType.LPWStr )] string pszAssemblyName,
			[In, Out] ref ASSEMBLY_INFO pAsmInfo );
		
		void CreateAssemblyCacheItem(
			uint dwFlags,
			IntPtr pvReserved,
			out IAssemblyCacheItem ppAsmItem,
			[MarshalAs( UnmanagedType.LPWStr ), Optional] string pszAssemblyName );
		
		void CreateAssemblyScavenger(
			[MarshalAs( UnmanagedType.Interface )] out object ppUnkReserved );
		
		void InstallAssembly(
			uint dwFlags,
			[MarshalAs( UnmanagedType.LPWStr )] string pszManifestFilePath,
			[In] ref FUSION_INSTALL_REFERENCE pRefData );
	}

	[ComImport]
	[InterfaceType( ComInterfaceType.InterfaceIsIUnknown )]
	[Guid( "9E3AAEB4-D1CD-11D2-BAB9-00C04F8ECEAE" )]
	internal interface IAssemblyCacheItem
	{
		void CreateStream(
			uint dwFlags,
			[MarshalAs( UnmanagedType.LPWStr )] string pszStreamName,
			uint dwFormat,
			uint dwFormatFlags,
			out IStream ppIStream,
			[In, Optional] ref _ULARGE_INTEGER puliMaxSize );
		
		void Commit(
			uint dwFlags,
			[Optional] out uint pulDisposition );
		
		void AbortItem();
	}

	[ComImport]
	[InterfaceType( ComInterfaceType.InterfaceIsIUnknown )]
	[Guid( "21B8916C-F28E-11D2-A473-00C04F8EF448" )]
	internal interface IAssemblyEnum
	{
		void GetNextAssembly(
			[Optional] IntPtr pvReserved,
			out IAssemblyName ppName,
			[Optional] uint dwFlags );
		
		void Reset();
		
		void Clone(
			out IAssemblyEnum ppEnum );
	}

	[ComImport]
	[InterfaceType( ComInterfaceType.InterfaceIsIUnknown )]
	[Guid( "CD193BC0-B4BC-11D2-9833-00C04FC31D2E" )]
	internal interface IAssemblyName
	{
		void SetProperty(
			uint PropertyId,
			IntPtr pvProperty,
			uint cbProperty );
		
		void GetProperty(
			uint PropertyId,
			IntPtr pvProperty,
			ref uint pcbProperty );

#pragma warning disable 465 // hides "Finalize" method name warning
        void Finalize();
#pragma warning restore 465

        void GetDisplayName(
			[MarshalAs( UnmanagedType.LPWStr )] StringBuilder szDisplayName,
			[In, Out] ref uint pccDisplayName,
			[Optional] ASM_DISPLAY_FLAGS dwDisplayFlags );

		void BindToObject(
			[In] ref Guid refIID,
			object pAsmBindSink,
			IApplicationContext pApplicationContext,
			[MarshalAs( UnmanagedType.LPWStr )] string szCodeBase,
			long llFlags,
			int pvReserved,
			uint cbReserved,
			out int ppv );

		void GetName(
			[In, Out] ref uint lpcwBuffer,
			[MarshalAs( UnmanagedType.LPWStr )] StringBuilder pwzName );

		void GetVersion(
			out uint pdwVersionHi,
			out uint pdwVersionLow );
		
		void IsEqual(
			IAssemblyName pName,
			uint dwCmpFlags );
		
		void Clone(
			out IAssemblyName pName );
	}

	[ComImport]
	[InterfaceType( ComInterfaceType.InterfaceIsIUnknown )]
	[Guid( "56B1A988-7C0C-4AA2-8639-C3EB5A90226F" )]
	internal interface IInstallReferenceEnum
	{
		void GetNextInstallReferenceItem(
			out IInstallReferenceItem ppRefItem,
			uint dwFlags,
			IntPtr pvReserved );
	}

	[ComImport]
	[InterfaceType( ComInterfaceType.InterfaceIsIUnknown )]
	[Guid( "582DAC66-E678-449F-ABA6-6FAAEC8A9394" )]
	internal interface IInstallReferenceItem
	{
		void GetReference(
			[MarshalAs( UnmanagedType.LPStruct )] out FUSION_INSTALL_REFERENCE ppRefData,
			uint dwFlags,
			IntPtr pvReserved );
	}

	[ComImport]
	[InterfaceType( ComInterfaceType.InterfaceIsIUnknown )]
	[Guid( "587BF538-4D90-4A3C-9EF1-58A200A8A9E7" )]
	internal interface IDefinitionIdentity
	{
		[SecurityCritical]
		[return: MarshalAs( UnmanagedType.LPWStr )]
		string GetAttribute(
			[MarshalAs( UnmanagedType.LPWStr ), In] string pszNamespace,
			[MarshalAs( UnmanagedType.LPWStr ), In] string pszName );

		[SecurityCritical]
		void SetAttribute(
			[MarshalAs( UnmanagedType.LPWStr ), In] string pszNamespace,
			[MarshalAs( UnmanagedType.LPWStr ), In] string pszName,
			[MarshalAs( UnmanagedType.LPWStr ), In] string pszValue );

		[SecurityCritical]
		IEnumIDENTITY_ATTRIBUTE EnumAttributes();

		[SecurityCritical]
		IDefinitionIdentity Clone(
			[In, Optional] IntPtr cDeltas,
			[MarshalAs( UnmanagedType.LPArray ), In, Optional] IDENTITY_ATTRIBUTE[] rgDeltas );
	}

	[ComImport]
	[InterfaceType( ComInterfaceType.InterfaceIsIUnknown )]
	[Guid( "6EAF5ACE-7917-4F3C-B129-E046A9704766" )]
    internal interface IReferenceIdentity
    {
		[SecurityCritical]
		[return: MarshalAs( UnmanagedType.LPWStr )]
		string GetAttribute(
			[MarshalAs( UnmanagedType.LPWStr ), In] string pszNamespace,
			[MarshalAs( UnmanagedType.LPWStr ), In] string pszName );

		[SecurityCritical]
		void SetAttribute(
			[MarshalAs( UnmanagedType.LPWStr ), In] string pszNamespace,
			[MarshalAs( UnmanagedType.LPWStr ), In] string pszName,
			[MarshalAs( UnmanagedType.LPWStr ), In] string pszValue );

		[SecurityCritical]
		IEnumIDENTITY_ATTRIBUTE EnumAttributes();

		[SecurityCritical]
		IReferenceIdentity Clone(
			[In, Optional] IntPtr cDeltas,
			[MarshalAs( UnmanagedType.LPArray ), In, Optional] IDENTITY_ATTRIBUTE[] rgDeltas );
    };

	[ComImport]
	[InterfaceType( ComInterfaceType.InterfaceIsIUnknown )]
	[Guid( "9CDAAE75-246E-4B00-A26D-B9AEC137A3EB" )]
    internal interface IEnumIDENTITY_ATTRIBUTE
    {
		[SecurityCritical]
		uint Next(
			[In] uint celt,
			[MarshalAs( UnmanagedType.LPArray ), Out] IDENTITY_ATTRIBUTE[] rgAttributes );

		[SecurityCritical]
		IntPtr CurrentIntoBuffer(
			[In] IntPtr cbAvailable,
			[MarshalAs( UnmanagedType.LPArray ), Out] byte[] Data );

		[SecurityCritical]
		void Skip(
			[In] uint celt );

		[SecurityCritical]
		void Reset();

		[SecurityCritical]
		IEnumIDENTITY_ATTRIBUTE Clone();
    };

	#endregion

	#region Enumerations

	// https://msdn.microsoft.com/en-us/library/ms231621%28v=vs.110%29.aspx

	[Flags]
	internal enum ASM_CACHE_FLAGS
	{
		None = 0,
		ZAP = 1, // Enumerates the cache of precompiled assemblies by using Ngen.exe.
		GAC = 2, // Enumerates the global assembly cache.
		DOWNLOAD = 4, // Enumerates the assemblies that have been downloaded on demand or that have been shadow-copied.
		ROOT = 8, // Indicates that the GetCachePath function should return the path to the global assembly cache for the common language runtime (CLR) version 2.0. Meaningful only in the context of a call to GetCachePath.
		ROOT_EX = 128, // Indicates that the GetCachePath function should return the path to the global assembly cache for CLR version 4. Meaningful only in the context of a call to GetCachePath.
	}

	[Flags]
	internal enum ASM_CMP_FLAGS
	{
		None = 0,
		NAME = 1,
		MAJOR_VERSION = 2,
		MINOR_VERSION = 4,
		BUILD_NUMBER = 8,
		REVISION_NUMBER = 16,
		VERSION = 30,
		PUBLIC_KEY_TOKEN = 32,
		CULTURE = 64,
		IL_NO_VERSION = 97,
		IL_ALL = 127,
		CUSTOM = 128,
		DEFAULT = 256,
		RETARGET = 512,
		ARCHITECTURE = 1024,
		CONFIG_MASK = 2048,
		MVID = 4096,
		SIGNATURE = 8192,
		CONTENT_TYPE = 16384,
	}

	[Flags]
	internal enum ASM_DISPLAY_FLAGS
	{
		None = 0,
		VERSION = 1,
		CULTURE = 2,
		PUBLIC_KEY_TOKEN = 4,
		PUBLIC_KEY = 8,
		CUSTOM = 16,
		PROCESSORARCHITECTURE = 32,
		LANGUAGEID = 64,
		RETARGET = 128,
		CONFIG_MASK = 256,
		MVID = 512,
		CONTENT_TYPE = 1024,
		FULL = VERSION | CULTURE | PUBLIC_KEY_TOKEN | PROCESSORARCHITECTURE | RETARGET | CONTENT_TYPE, // 1191
	}

	internal enum ASM_NAME
	{
		PUBLIC_KEY = 0,
		PUBLIC_KEY_TOKEN,
		HASH_VALUE,
		NAME,
		MAJOR_VERSION,
		MINOR_VERSION,
		BUILD_NUMBER,
		REVISION_NUMBER,
		CULTURE,
		PROCESSOR_ID_ARRAY,
		OSINFO_ARRAY,
		HASH_ALGID,
		ALIAS,
		CODEBASE_URL,
		CODEBASE_LASTMOD,
		NULL_PUBLIC_KEY,
		NULL_PUBLIC_KEY_TOKEN,
		CUSTOM,
		NULL_CUSTOM,
		MVID,
		FILE_MAJOR_VERSION,
		FILE_MINOR_VERSION,
		FILE_BUILD_NUMBER,
		FILE_REVISION_NUMBER,
		RETARGET,
		SIGNATURE_BLOB,
		CONFIG_MASK,
		ARCHITECTURE,
		CONTENT_TYPE,
		MAX_PARAMS,
	}

	internal enum AssemblyComparisonResult
	{
		Unknown = 0,
		EquivalentFullMatch,
		EquivalentWeakNamed,
		EquivalentFXUnified,
		EquivalentUnified,
		NonEquivalentVersion,
		NonEquivalent,
		EquivalentPartialMatch,
		EquivalentPartialWeakNamed,
		EquivalentPartialUnified,
		EquivalentPartialFXUnified,
		NonEquivalentPartialVersion,
	}

	internal enum AssemblyContentType
	{
		Invalid = -1,
		Default = 0,
		WindowsRuntime = 1,
	}

	[Flags]
	internal enum AssemblyInfo_Flag
	{
		None = 0,
		Installed = 1,
		PayloadResident = 2
	}

	[Flags]
	internal enum CREATE_ASM_NAME_OBJ_FLAGS
	{
		None = 0,
		PARSE_DISPLAY_NAME = 1,
		SET_DEFAULT_VALUES = 2,
		VERIFY_FRIEND_ASSEMBLYNAME = 4,
		PARSE_FRIEND_DISPLAY_NAME = 5,
	}

	[Flags]
	internal enum IAssemblyCache_InstallFlag
	{
		None = 0,
		Refresh = 0x01,
		ForceRefresh = 0x02
	}

	internal enum IAssemblyCache_UninstallDisposition
	{
		None = 0,
		Uninstalled = 1,
		StillInUse = 2,
		AlreadyUninstalled = 3,
		DeletePending = 4,
		HasInstallReferences = 5,
		ReferenceNotFound = 6
	}

	internal enum IAssemblyCacheItem_CommitDisposition
	{
		None = 0,
		Installed = 1,
		Refreshed = 2,
		AlreadyInstalled = 3,
	}

	internal enum IAssemblyCacheItem_CommitFlag
	{
		None = 0,
		Refresh = 1,
		ForceRefresh = 2,
	}

	internal enum PEKIND
	{
		peInvalid = -1,
		peNone = 0,
		peMSIL = 1,
		peI386 = 2,
		peIA64 = 3,
		peAMD64 = 4,
		peARM = 5,
	}

	[Flags]
	internal enum QueryAsmInfo_Flag
	{
		None = 0,
		Validate = 1,
		GetSize = 2
	}

	[Flags]
	internal enum StreamFormat
	{
		CompLibModule = 0,
		CompLibManifest = 1,
		Win32Module = 2,
		Win32Manifest = 4,
	}

	#endregion

	#region Structures

	[StructLayout( LayoutKind.Sequential )]
	internal struct _FILETIME
	{
		public uint dwLowDateTime;
		public uint dwHighDateTime;
	}

	[StructLayout( LayoutKind.Sequential )]
	internal struct _LARGE_INTEGER
	{
		public long QuadPart;
	}

	[StructLayout( LayoutKind.Sequential )]
	internal struct _ULARGE_INTEGER
	{
		public ulong QuadPart;
	}

	[StructLayout( LayoutKind.Sequential, CharSet = CharSet.Unicode )]
	internal struct ASSEMBLY_INFO
	{
		public int structSize;
		public AssemblyInfo_Flag assemblyFlags;
		public long assemblySizeInKB;
		[MarshalAs( UnmanagedType.LPWStr )]
		public string currentAssemblyPath;
		public int bufferLength;

		public ASSEMBLY_INFO( int bufferLength )
			: this()
		{
			structSize = Marshal.SizeOf( typeof( ASSEMBLY_INFO ) );
			currentAssemblyPath = new String( '\0', bufferLength );
			this.bufferLength = bufferLength;
		}
	}

	[StructLayout( LayoutKind.Sequential, CharSet = CharSet.Unicode )]
	internal struct FUSION_INSTALL_REFERENCE
	{
		public int structSize;
		public int flags;
		public Guid scheme;
		[MarshalAs( UnmanagedType.LPWStr )]
		public string identifier;
		[MarshalAs( UnmanagedType.LPWStr )]
		public string nonCannonicalData;
	}

	[StructLayout( LayoutKind.Sequential, CharSet = CharSet.Unicode )]
	internal struct IDENTITY_ATTRIBUTE
	{
		[MarshalAs( UnmanagedType.LPWStr )]
		public string Namespace;
		[MarshalAs( UnmanagedType.LPWStr )]
		public string Name;
		[MarshalAs( UnmanagedType.LPWStr )]
		public string Value;

		public override string ToString()
		{
			var sb = new StringBuilder( "{IDENTITY_ATTRIBUTE:" );
			sb.Append( " Namespace=" ).Append( Namespace != null ? Namespace : "(null)" ).Append( "," );
			sb.Append( " Name=" ).Append( Name );
			sb.Append( ", Value=" ).Append( Value );
			sb.Append( "}" );
			return sb.ToString();
		}
	};

	#endregion
}
