using System;
using System.Collections;
using System.Collections.Generic;
using System.Net;
using System.Net.Sockets;
using System.Reflection;
using System.Threading;

namespace IntelliSite.CommonUtil
{
	/// <summary>
	/// Summary description for MessageSocket.
	/// </summary>

	//**********************************************************************************************************************************
	//*                                                                                                                                *
	//* This class is obsolete!  It has bugs if the DataStream goes too fast.  Use PeekableSocketClientStream + MessageStream instead! *
	//*                                                                                                                                *
	//**********************************************************************************************************************************

	public class MessageFrom
	{
		public SocketState SocketState { get; private set; }
		public MessageBase      Message { get; private set; }
		
		public MessageFrom( SocketState socketState, MessageBase message )
		{
			SocketState	= socketState;
			Message		= message;
		}
	}
	
	public delegate void SocketConnectionLoss( SocketState socketState, Exception e );

	public class MessageSocket : IDisposable
	{
		public bool                     Listening            { get; private set; }
		public SocketState              SocketState          { get; private set; }
		public List<SocketState>        AcceptedSockets      { get; private set; }
		public AutoResetEvent           FromRemoteEvent      { get; private set; }
		public Queue<MessageFrom>       FromRemoteQueue      { get; private set; }
		public string                   Name                 { get; set; }
		public LoggerBase               Logger               { get; private set; }
		public SocketConnectionLoss     SocketConnectionLoss { get; set; }
		public static MessageDictionary MessageDictionary    { get; private set; }
		
		public MessageSocket( string name, LoggerBase logger )
		{
			Name	                = name;
			Logger	                = logger;
			AcceptedSockets			= new List<SocketState>();
			FromRemoteEvent			= new AutoResetEvent( false );
			FromRemoteQueue			= new Queue<MessageFrom>();
			SocketConnectionLoss	= null;
			SocketState             = null;

			if( MessageDictionary == null )
				MessageDictionary = new MessageDictionary();
		}

		#region Destructor/Dispose
		~MessageSocket()
		{
			Dispose( false );
		}

		public void Dispose()
		{
			Dispose( true );
		}

		private bool disposed = false;
		protected void Dispose( bool disposing )
		{
			if( !disposed )
			{
				if( disposing )
					Close();
			}
		}
		#endregion		

		public bool Connected { get { return SocketState != null && SocketState.Socket != null && SocketState.Socket.Connected; } }
		
		public bool Open( string address, int port, SocketType socketType, ProtocolType protocolType )
		{
			if( Logger != null )
			{
				string target = " (";
				if( address != null )
					target += address + ", ";
				target += port + " " + socketType + " " + protocolType + ")";

				Logger.Write( Name + " Opening" + target );
			}

			Listening = address == null;
			
			try
			{
				Socket socket = new Socket( AddressFamily.InterNetwork, socketType, protocolType );
				SocketState = new SocketState( this, socket );

				if( Listening )
				{
					IPEndPoint endPoint = new IPEndPoint( IPAddress.Any, port );
					socket.Bind( endPoint );
					socket.Listen( 50 );

					if( Logger != null )
						Logger.Write( Name + " Listening" );
					socket.BeginAccept( new AsyncCallback( AcceptCallback ), SocketState );
					return true;
				}
				else
				{
					IPEndPoint endPoint = SimpleSocket.GetIPEndPoint( address, port, Logger );
					socket.Connect( endPoint );
					if( !socket.Connected ) 
					{
						SocketState.LastError = Convert.ToString( System.Runtime.InteropServices.Marshal.GetLastWin32Error() );
						Console.WriteLine( "Winsock error: " + SocketState.LastError );
						
						if( Logger != null )
							Logger.Write( Name + " Open error: " + SocketState.LastError );
						return false;
					}

					if( Logger != null )
						Logger.Write( Name + " Opened" );
						
					socket.BeginReceive( SocketState.FromRemoteBuffer1, 0, SocketState.FromRemoteBuffer1.Length, SocketFlags.None, new AsyncCallback( ReceiveCallback ), SocketState );
					return true;
				}
			}
			catch( Exception e ) 
			{
				SocketState.LastError = e.ToString();
				if( Logger != null )
					Logger.Write( Name + " Open exception: " + SocketState.LastError );
				return false;
			}
		}

		public void Close()
		{
			if( Logger != null )
				Logger.Write( Name + " Closing" );

			if( SocketState != null )
			{
				if( SocketState.Socket != null )
					SocketState.Socket.Dispose();
				SocketState = null;
			}

			if( AcceptedSockets.Count > 0 )
				AcceptedSockets.RemoveRange( 0, AcceptedSockets.Count );

			if( Logger != null )
				Logger.Write( Name + " Closed" );
		}

		private void RemoveSocketState( SocketState delSocketState )
		{
			if( delSocketState == SocketState )
				Close();
			else
				AcceptedSockets.Remove( delSocketState );
		}

		private MessageFrom GetNextMessage()
		{
			lock( FromRemoteQueue )
			{
				MessageFrom messageFrom = null;
				if( FromRemoteQueue.Count > 0 )
					messageFrom = (MessageFrom)FromRemoteQueue.Dequeue();

				if( messageFrom.Message.Type == DisconnectMessage.MessageType )
					AcceptedSockets.Remove( messageFrom.SocketState );	

				return messageFrom;
			}
		}

		public void SendAll( MessageBase message, byte type )
		{
			if( Logger != null )
				Logger.Write( Name + " SendAll " + type + " (message: " + message.Type + ") Enter" );

			lock( AcceptedSockets )
			{
				for( int i=0; i<AcceptedSockets.Count; i++ )
				{
					SocketState acceptedState = (SocketState)AcceptedSockets[i];
					if( type == 0 || acceptedState.Type == type )
						Send( acceptedState, message );
				}	
			}

			if( Logger != null )
				Logger.Write( Name + " SendAll " + type + " (message: " + message.Type + ") Exit" );
		}

		public void Send( MessageBase message )
		{
			Send( SocketState, message );
		}

		public void Send( SocketState txSocketState, MessageBase message )
		{
			if( Logger != null )
				Logger.Write( Name + " SendTo (message: " + message.Type + " data size: " + message.DataLen + ") Enter" );

			lock( txSocketState.ToRemoteQueue )
			{
				txSocketState.ToRemoteQueue.Enqueue( message );
				if( !txSocketState.IsSendActive )
				{
					int size;
					message.GetNetworkBuffer( out size, txSocketState.ToRemoteBuffer );

					try
					{
						if( Logger != null )
							Logger.Write( Name + " SendTo (sent " + size + " bytes)" );

						txSocketState.Socket.BeginSend( txSocketState.ToRemoteBuffer, 0, size, SocketFlags.None, new AsyncCallback( SendCallback ), txSocketState );
						txSocketState.IsSendActive = true;
					}
					catch (Exception e)
					{
						txSocketState.LastError = e.ToString();
						Console.WriteLine( "Winsock error: " + SocketState.LastError );
						if( Logger != null )
							Logger.Write( Name + " SendTo Exception: " + SocketState.LastError );

						if( txSocketState.Parent.SocketConnectionLoss != null )
							txSocketState.Parent.SocketConnectionLoss( txSocketState, e );

						AcceptedSockets.Remove( txSocketState );
					}
				}
			}

			if( Logger != null )
				Logger.Write( Name + " SendTo (message: " + message.Type + ") Exit" );
		}

		private static void AcceptCallback( IAsyncResult ar ) 
		{
			SocketState socketState = (SocketState) ar.AsyncState;
			lock( socketState.Parent.AcceptedSockets )
			{
				try 
				{
					SocketState acceptedSocketState = new SocketState( socketState.Parent, socketState.Socket.EndAccept( ar ) );
					socketState.Parent.AcceptedSockets.Add( acceptedSocketState );

					if( socketState.Parent.Logger != null )
						socketState.Parent.Logger.Write( socketState.Parent.Name + " Accepted: " + acceptedSocketState.Socket.RemoteEndPoint );

					acceptedSocketState.Socket.BeginReceive( acceptedSocketState.FromRemoteBuffer1, 0,  socketState.FromRemoteBuffer1.Length, SocketFlags.None, new AsyncCallback( ReceiveCallback ), acceptedSocketState );
					socketState.Socket.BeginAccept( new AsyncCallback( AcceptCallback ), socketState );			
				} 
				catch (Exception e) 
				{
					socketState.LastError = e.ToString();
					Console.WriteLine( "Winsock error: " + socketState.LastError );

					if( socketState.Parent.Logger != null )
						socketState.Parent.Logger.Write( socketState.Parent.Name + " Accept Exception: " + socketState.LastError );

					if( socketState.Parent.SocketConnectionLoss != null )
						socketState.Parent.SocketConnectionLoss( socketState, e );

					socketState.Parent.AcceptedSockets.Remove( socketState );
				}
			}
		}

		private static void ReceiveCallback( IAsyncResult ar ) 
		{
			SocketState rxSocketState = (SocketState) ar.AsyncState;
			lock( rxSocketState )
			{
				try 
				{
					int bytes = rxSocketState.Socket.EndReceive( ar );
					if( bytes == 0 )
					{
						if( rxSocketState.Socket.Available == 0 && rxSocketState.Socket.Poll( 1, SelectMode.SelectRead ) )
						{
							if( rxSocketState.Parent.Logger != null )
								rxSocketState.Parent.Logger.Write( rxSocketState.Parent.Name + " Socket closed." );

							if( rxSocketState.Parent.Listening )
								rxSocketState.Parent.AcceptedSockets.Remove( rxSocketState );

							if( rxSocketState.Parent.SocketConnectionLoss != null )
								rxSocketState.Parent.SocketConnectionLoss( rxSocketState, new Exception() );
							return;
						}
					}

					if( rxSocketState.Parent.Logger != null )
						rxSocketState.Parent.Logger.Write( String.Format( "{0} received {1} bytes into {2}", rxSocketState.Parent.Name, bytes, rxSocketState.IsBuffer1 ? "Buffer 1" : "Buffer 2" ) );

					byte [] buffer = rxSocketState.IsBuffer1 ? rxSocketState.FromRemoteBuffer1 : rxSocketState.FromRemoteBuffer2;

				//	Commenting this out and moving it until after I process a packet because lock() doesn't lock if the calls come from the same thread,
				//	and BeginAsync will complete on the same thread and not block.  This is really, really stupid.
				//
				//	rxSocketState.IsBuffer1 = !rxSocketState.IsBuffer1;
				//	rxSocketState.Socket.BeginReceive( rxSocketState.IsBuffer1 ? rxSocketState.FromRemoteBuffer1 : rxSocketState.FromRemoteBuffer2, 0,  rxSocketState.FromRemoteBuffer1.Length, SocketFlags.None, new AsyncCallback( ReceiveCallback ), rxSocketState );

					int used	= 0;
				
					while( used < bytes )
					{
						int beginning = used;
						if( rxSocketState.FromRemoteMessage == null )
						{
							rxSocketState.FromRemoteMessage = new MessageBase();

							if( rxSocketState.Parent.Logger != null )
								rxSocketState.Parent.Logger.Write( rxSocketState.Parent.Name + " New message" );
						}
					
						if( rxSocketState.FromRemoteMessage.ReceiveNetworkBuffer( buffer, bytes, ref used ) )
						{
							if( rxSocketState.Parent.Logger != null )
								rxSocketState.Parent.Logger.Write( String.Format( "{0} Queueing message, used {1} bytes, {2} remaining", rxSocketState.Parent.Name, used - beginning, bytes - used ) );

							MessageBase specificMessage = (MessageBase)MessageDictionary.GetMessageOfType( rxSocketState.FromRemoteMessage );
							MessageFrom messageFrom = new MessageFrom( rxSocketState, specificMessage );

							if( rxSocketState.Parent.Logger != null )
								rxSocketState.Parent.Logger.Write( rxSocketState.Parent.Name + " Queued message: " + specificMessage );

							lock( rxSocketState.Parent.FromRemoteQueue )
							{
								rxSocketState.Parent.FromRemoteQueue.Enqueue( messageFrom );
							}
							rxSocketState.Parent.FromRemoteEvent.Set();
					
							rxSocketState.FromRemoteMessage = null;
						}
						else if( rxSocketState.Parent.Logger != null )
							rxSocketState.Parent.Logger.Write( String.Format( "{0} used {1} bytes, {2} remaining", rxSocketState.Parent.Name, used - beginning, bytes - used ) );
					}

					rxSocketState.IsBuffer1 = !rxSocketState.IsBuffer1;
					rxSocketState.Socket.BeginReceive( rxSocketState.IsBuffer1 ? rxSocketState.FromRemoteBuffer1 : rxSocketState.FromRemoteBuffer2, 0, rxSocketState.FromRemoteBuffer1.Length, SocketFlags.None, new AsyncCallback( ReceiveCallback ), rxSocketState );

				} 
				catch (Exception e) 
				{
					if( rxSocketState.Parent.Listening )
						rxSocketState.Parent.AcceptedSockets.Remove( rxSocketState );
			
					rxSocketState.LastError = e.ToString();
					if( rxSocketState.Parent.Logger != null )
						rxSocketState.Parent.Logger.Write( rxSocketState.Parent.Name + " Receive Exception: " +  rxSocketState.LastError );

					if( rxSocketState.Parent.SocketConnectionLoss != null )
						rxSocketState.Parent.SocketConnectionLoss( rxSocketState, e );

					rxSocketState.Parent.AcceptedSockets.Remove( rxSocketState );
				}
			}
		}

		private static void SendCallback( IAsyncResult ar ) 
		{
			SocketState txSocketState = (SocketState) ar.AsyncState;
			lock( txSocketState.ToRemoteQueue )
			{
				try 
				{
					if( txSocketState.IsSendActive )
					{
						txSocketState.Socket.EndSend( ar );
						MessageBase message = (MessageBase)txSocketState.ToRemoteQueue.Peek();
						if( message.NetworkTransferComplete )
						{
							message.Transferred = 0;
							if( txSocketState.Parent.Logger != null )
								txSocketState.Parent.Logger.Write( txSocketState.Parent.Name + " Send Complete: (message: " + message.Type + ")" );

							txSocketState.ToRemoteQueue.Dequeue();
							txSocketState.IsSendActive = false;
						}
					}
						
					if( txSocketState.ToRemoteQueue.Count > 0 )
					{
						MessageBase message = (MessageBase)txSocketState.ToRemoteQueue.Peek();
						int size;
						if( txSocketState.IsSendActive )
						{
							message.GetNextNetworkBuffer( out size, txSocketState.ToRemoteBuffer );
							txSocketState.Socket.BeginSend( txSocketState.ToRemoteBuffer, 0, size, SocketFlags.None, new AsyncCallback( SendCallback ), txSocketState );

							if( txSocketState.Parent.Logger != null )
								txSocketState.Parent.Logger.Write( txSocketState.Parent.Name + " Tx: " + size + " bytes, " + message.Transferred + " of " + message.Data.Length );
						}
						else
						{
							message.GetNetworkBuffer( out size, txSocketState.ToRemoteBuffer );
							txSocketState.Socket.BeginSend( txSocketState.ToRemoteBuffer, 0, size, SocketFlags.None, new AsyncCallback( SendCallback ), txSocketState );

							if( txSocketState.Parent.Logger != null )
								txSocketState.Parent.Logger.Write( txSocketState.Parent.Name + " Tx Start: " + size + " bytes" );
						}
					
						txSocketState.IsSendActive = true;
					} 
				}
				catch (Exception e) 
				{
					if( txSocketState.Parent.Listening )
						txSocketState.Parent.AcceptedSockets.Remove( txSocketState );
			
					txSocketState.LastError = e.ToString();
					Console.WriteLine( "Winsock error: " + txSocketState.LastError );

					if( txSocketState.Parent.Logger != null )
						txSocketState.Parent.Logger.Write( txSocketState.Parent.Name + " Send Exception: " + txSocketState.LastError );

					if( txSocketState.Parent.SocketConnectionLoss != null )
						txSocketState.Parent.SocketConnectionLoss( txSocketState, e );

					txSocketState.Parent.AcceptedSockets.Remove( txSocketState );
				}
			}
		}
	}
}
