﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Sockets;
using System.Text;

namespace IntelliSite.CommonUtil
{
	public class MessageSocketServer : IDisposable
	{
		public string                        Name                   { get; set; }
		public Encoding                      Encoding               { get; set; }
		public LoggerBase                    Logger                 { get; set; }
		public List<MessageStream>           MessageStreams         { get; private set; }
		public SocketServer                  SocketServer           { get; private set; }
		public EventHandler<EventArgs>       MessageReceivedHandler { get; set; }

		public event EventHandler<ErrorArgs> ConnectionLoss;

		public MessageSocketServer( string name, EventHandler<EventArgs> messageReceivedHandler, Encoding encoding = null, LoggerBase logger = null )
		{
			Name                             = name;
			MessageReceivedHandler           = messageReceivedHandler;
			Encoding                         = encoding;
			Logger                           = logger;
			MessageStreams                   = new List<MessageStream>();
			SocketServer                     = new SocketServer( "MessageSocketServer", encoding, logger );
			SocketServer.ConnectionAccepted += ConnectionAccepted;
		}

		~MessageSocketServer()
		{
			Dispose( false );
		}

		public void Dispose()
		{
			Dispose( true );
		}

		private bool disposed = false;
		protected void Dispose( bool disposing )
		{
			if( !disposed )
			{
				if( disposing )
				{
					while( MessageStreams.Count > 0 )
						OnClientConnectionLoss( MessageStreams[0].Stream, new ErrorArgs( this, "Shutdown" ) );
					MessageStreams.Clear();

					if( SocketServer != null )
						SocketServer.Dispose();
				}
			}
		}

		public void Close()
		{
			SocketServer.Close();
		}

		public bool Listening { get { return SocketServer != null && SocketServer.Listening; } }

		public bool Listen( int port,
							bool waitForCompletion = true,
							AddressFamily addressFamily = AddressFamily.InterNetwork,
							SocketType socketType = SocketType.Stream,
							ProtocolType protocolType = ProtocolType.Tcp )
		{
			return SocketServer.Listen( port, waitForCompletion, addressFamily, socketType, protocolType );
		}

		public void CloseMessageStream( MessageStream stream )
		{
			OnClientConnectionLoss( stream, new ErrorArgs( this, "CloseMessageStream" ) );
		}

		private void ConnectionAccepted( object sender, ConnectionArgs args )
		{
			MessageStreams.Add( new MessageStream( "New Message Stream", MessageReceivedHandler, args.Stream, Logger, this ) );
		}

		private void OnClientConnectionLoss( object sender, ErrorArgs args )
		{
			foreach( MessageStream messageStream in MessageStreams )
				if( messageStream == sender as MessageStream || messageStream.Stream == sender as PeekableSocketClientStream )
				{
					SocketServer.OnClientConnectionLoss( messageStream.Stream, args );

					MessageStreams.Remove( messageStream );

					if( ConnectionLoss != null )
						ConnectionLoss( messageStream, args );

					messageStream.Dispose();
					break;
				}
		}
	}
}
