﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;

namespace IntelliSite.CommonUtil
{
	public class PeekableMemoryStream : PeekableStream
	{
		public IList<byte[]> DataList  { get; set; }
		public byte[]        DataArray { get; set; }

		public PeekableMemoryStream( IList<byte[]> dataList ) : base()
		{
			DataList  = dataList;
			DataArray = null;
		}

		public PeekableMemoryStream( byte[] dataArray ) : base()
		{
			DataArray = dataArray;
			DataList  = null;
		}

		public override bool SupportsMultipleChunks                         { get { return DataList != null; } }

		public override int ChunkCount                                      { get { return DataList != null ? DataList.Count : 1; } }
		public override int ChunkSize( int index )	                        { return DataList != null ? DataList[index].Length : DataArray.Length; }
		public override byte[] PeekChunk( int index, int offset, int size )
		{
			if( DataList != null )
				return DataList[index];
			else if( index == 0 && ( ( offset + size ) <= DataArray.Length ) )
			{
				byte [] data = new byte[size];
				Read( data, offset, size );
				return data;
			}
			else
				throw new IndexOutOfRangeException();
		}

		public override bool CanRead                                        { get { return true; } }
		public override bool CanSeek                                        { get { return false; } }
		public override bool CanWrite                                       { get { return true; } }
		public override void Flush()                                        { throw new NotImplementedException(); }
		public override long Position                                       { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }
		public override void SetLength( long value )                        { throw new NotImplementedException(); }
		public override long Seek( long offset, SeekOrigin origin )			{ throw new NotImplementedException(); }

		public override int Read( byte[] buffer, int offset, int count )
		{
			if( !SupportsMultipleChunks )
			{
				Array.Copy( DataArray, offset, buffer, 0, count );
				return count;
			}
			else
				throw new NotImplementedException();
		}

		public override void Write( byte[] buffer, int offset, int count )
		{
			throw new NotImplementedException();
		}
	}
}
