﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using Microsoft.Win32;

namespace IntelliSite.CommonUtil
{
	public class ProductComponentInfo : IEquatable<ProductComponentInfo>
	{
		protected ProductComponentInfo()
		{
		}

		public ProductComponentInfo( string groupKey, string itemKey, RegistryKey key )
		{
			this.GroupKey            = groupKey;
			this.ItemKey             = itemKey;
			this.IsInstalled         = (int)key.GetValue( "Installed", 0 ) != 0;
			this.Folder              = (string)key.GetValue( "Folder" );
			this.DisplayName         = (string)key.GetValue( "DisplayName" );
			this.AssemblyNameString  = (string)key.GetValue( "AssemblyName" );
			this.AssemblyPath        = (string)key.GetValue( "AssemblyPath" );
			this.VersionString       = (string)key.GetValue( "Version" );

			if( !String.IsNullOrEmpty( this.AssemblyNameString ) )
				this.AssemblyName = new AssemblyName( this.AssemblyNameString );

			if( !String.IsNullOrEmpty( this.VersionString ) )
			{
				Version version;
				if( Version.TryParse( this.VersionString, out version ) )
					this.Version = version;
			}
		}

		public virtual string GroupKey { get; protected set; }
		public virtual string ItemKey { get; protected set; }
		public virtual bool IsInstalled { get; protected set; }
		public virtual string Folder { get; protected set; }
		public virtual string DisplayName { get; protected set; }
		public virtual AssemblyName AssemblyName { get; protected set; }
		public virtual string AssemblyNameString { get; protected set; }
		public virtual string AssemblyPath { get; protected set; }
		public virtual Version Version { get; protected set; }
		public virtual string VersionString { get; protected set; }

		public bool Equals( ProductComponentInfo other )
		{
			return
				this.GroupKey.Equals( other.GroupKey, StringComparison.InvariantCulture ) &&
				this.ItemKey.Equals( other.ItemKey, StringComparison.InvariantCulture ) &&
				this.IsInstalled == other.IsInstalled &&
				this.Folder.Equals( other.Folder, StringComparison.InvariantCulture ) &&
				this.DisplayName.Equals( other.DisplayName, StringComparison.InvariantCulture ) &&
				this.AssemblyName == other.AssemblyName &&
				this.AssemblyNameString.Equals( other.AssemblyNameString, StringComparison.InvariantCulture ) &&
				this.AssemblyPath.Equals( other.AssemblyPath, StringComparison.InvariantCulture ) &&
				this.Version == other.Version &&
				this.VersionString.Equals( other.VersionString, StringComparison.InvariantCulture );
		}

		public bool KeyEquals( ProductComponentInfo other )
		{
			return
				this.GroupKey.Equals( other.GroupKey, StringComparison.InvariantCulture ) &&
				this.ItemKey.Equals( other.ItemKey, StringComparison.InvariantCulture );
		}
	}
}
