﻿// -----------------------------------------------------------------------
// <copyright file="FrameworkSerializer.cs" company="">
// TODO: Update copyright text.
// </copyright>
// -----------------------------------------------------------------------

namespace IntelliSite.CommonUtil.Serializers.FrameworkSerializers
{
	using System;
	using System.Collections.Generic;
	using System.Linq;
	using System.Text;
	using System.Windows;
	using System.Windows.Media;

	/// <summary>
	/// Serialize a System.Windows.Media.Color struct
	/// </summary>
	[Serializer( typeof( Color ) )]
	public class SerializeColor : ISerializeObject
	{
		public object[] Serialize( object target )
		{
			Color color = (Color)target;
			return new object[] { color.A, color.R, color.G, color.B };
		}

		public object Deserialize( object[] data )
		{
			byte a = (byte)data[0];
			byte r = (byte)data[1];
			byte g = (byte)data[2];
			byte b = (byte)data[3];
			return Color.FromArgb( a, r, g, b );
		}
	}

	/// <summary>
	/// Serialize a System.Windows.Point struct
	/// </summary>
	[Serializer( typeof( Point ) )]
	public class SerializePoint : ISerializeObject
	{
		public object[] Serialize( object target )
		{
			Point point = (Point)target;
			return new object[] { point.X, point.Y };
		}

		public object Deserialize( object[] data )
		{
			double x = (double)data[0];
			double y = (double)data[1];
			return new Point( x, y );
		}
	}

	/// <summary>
	/// Serialize a System.Windows.Rect struct
	/// </summary>
	[Serializer( typeof( Rect ) )]
	public class SerializeRect : ISerializeObject
	{
		public object[] Serialize( object target )
		{
			Rect rect = (Rect)target;
			return new object[] { rect.X, rect.Y, rect.Width, rect.Height };
		}

		public object Deserialize( object[] data )
		{
			double x = (double)data[0];
			double y = (double)data[1];
			double width = (double)data[2];
			double height = (double)data[3];
			return new Rect( x, y, width, height );
		}
	}

	/// <summary>
	/// Serialize a System.Windows.FontFamily struct
	/// </summary>
	[Serializer( typeof( FontFamily ) )]
	public class SerializeFontFamily : ISerializeObject
	{
		public object[] Serialize( object target )
		{
			FontFamily family = (FontFamily)target;
			FontFamilyConverter converter = new FontFamilyConverter();
			return new object[] { converter.ConvertToInvariantString( family ) };
		}

		public object Deserialize( object[] data )
		{
			FontFamilyConverter converter = new FontFamilyConverter();
			string converted = (string)data[0];
			FontFamily family = (FontFamily)converter.ConvertFromInvariantString( converted );
			return family;
		}
	}

	/// <summary>
	/// Serialize a System.Windows.FontWeight struct
	/// </summary>
	[Serializer( typeof( FontWeight ) )]
	public class SerializeFontWeight : ISerializeObject
	{
		public object[] Serialize( object target )
		{
			FontWeight weight = (FontWeight)target;
			FontWeightConverter converter = new FontWeightConverter();
			return new object[] { converter.ConvertToInvariantString( weight ) };
		}

		public object Deserialize( object[] data )
		{
			FontWeightConverter converter = new FontWeightConverter();
			string converted = (string)data[0];
			FontWeight weight = (FontWeight)converter.ConvertFromInvariantString( converted );
			return weight;
		}
	}

	/// <summary>
	/// Serialize a System.Windows.FontStyle struct
	/// </summary>
	[Serializer( typeof( FontStyle ) )]
	public class SerializeFontStyle : ISerializeObject
	{
		public object[] Serialize( object target )
		{
			FontStyle style = (FontStyle)target;
			FontStyleConverter converter = new FontStyleConverter();
			return new object[] { converter.ConvertToInvariantString( style ) };
		}

		public object Deserialize( object[] data )
		{
			FontStyleConverter converter = new FontStyleConverter();
			string converted = (string)data[0];
			FontStyle style = (FontStyle)converter.ConvertFromInvariantString( converted );
			return style;
		}
	}

	/// <summary>
	/// Serialize a System.Windows.FontStretch struct
	/// </summary>
	[Serializer( typeof( FontStretch ) )]
	public class SerializeFontStretch : ISerializeObject
	{
		public object[] Serialize( object target )
		{
			FontStretch stretch = (FontStretch)target;
			FontStretchConverter converter = new FontStretchConverter();
			return new object[] { converter.ConvertToInvariantString( stretch ) };
		}

		public object Deserialize( object[] data )
		{
			FontStretchConverter converter = new FontStretchConverter();
			string converted = (string)data[0];
			FontStretch stretch = (FontStretch)converter.ConvertFromInvariantString( converted );
			return stretch;
		}
	}
}
