﻿using System;

namespace IntelliSite.CommonUtil
{
	public class SocketErrorString
	{
		public static string FullError( int errorCode )
		{
			return String.Format( "{0}, {1}", ShortError( errorCode ), LongDescription( errorCode ) );
		}

		public static string ShortError( int errorCode )
		{
			switch( errorCode )
			{
				case 6:		return "WSA_INVALID_HANDLE";
				case 8:		return "WSA_NOT_ENOUGH_MEMORY";
				case 87:	return "WSA_INVALID_PARAMETER";
				case 995:	return "WSA_OPERATION_ABORTED";
				case 996:	return "WSA_IO_INCOMPLETE";
				case 997:	return "WSA_IO_PENDING";
				case 10004:	return "WSAEINTR";
				case 10009:	return "WSAEBADF";
				case 10013:	return "WSAEACCES";
				case 10014:	return "WSAEFAULT";
				case 10022:	return "WSAEINVAL";
				case 10024:	return "WSAEMFILE";
				case 10035:	return "WSAEWOULDBLOCK";
				case 10036:	return "WSAEINPROGRESS";
				case 10037:	return "WSAEALREADY";
				case 10038:	return "WSAENOTSOCK";
				case 10039:	return "WSAEDESTADDRREQ";
				case 10040:	return "WSAEMSGSIZE";
				case 10041:	return "WSAEPROTOTYPE";
				case 10042:	return "WSAENOPROTOOPT";
				case 10043:	return "WSAEPROTONOSUPPORT";
				case 10044:	return "WSAESOCKTNOSUPPORT";
				case 10045:	return "WSAEOPNOTSUPP";
				case 10046:	return "WSAEPFNOSUPPORT";
				case 10047:	return "WSAEAFNOSUPPORT";
				case 10048:	return "WSAEADDRINUSE";
				case 10049:	return "WSAEADDRNOTAVAIL";
				case 10050:	return "WSAENETDOWN";
				case 10051:	return "WSAENETUNREACH";
				case 10052:	return "WSAENETRESET";
				case 10053:	return "WSAECONNABORTED";
				case 10054:	return "WSAECONNRESET";
				case 10055:	return "WSAENOBUFS";
				case 10056:	return "WSAEISCONN";
				case 10057:	return "WSAENOTCONN";
				case 10058:	return "WSAESHUTDOWN";
				case 10059:	return "WSAETOOMANYREFS";
				case 10060:	return "WSAETIMEDOUT";
				case 10061:	return "WSAECONNREFUSED";
				case 10062:	return "WSAELOOP";
				case 10063:	return "WSAENAMETOOLONG";
				case 10064:	return "WSAEHOSTDOWN";
				case 10065:	return "WSAEHOSTUNREACH";
				case 10066:	return "WSAENOTEMPTY";
				case 10067:	return "WSAEPROCLIM";
				case 10068:	return "WSAEUSERS";
				case 10069:	return "WSAEDQUOT";
				case 10070:	return "WSAESTALE";
				case 10071:	return "WSAEREMOTE";
				case 10091:	return "WSASYSNOTREADY";
				case 10092:	return "WSAVERNOTSUPPORTED";
				case 10093:	return "WSANOTINITIALISED";
				case 10101:	return "WSAEDISCON";
				case 10102:	return "WSAENOMORE";
				case 10103:	return "WSAECANCELLED";
				case 10104:	return "WSAEINVALIDPROCTABLE";
				case 10105:	return "WSAEINVALIDPROVIDER";
				case 10106:	return "WSAEPROVIDERFAILEDINIT";
				case 10107:	return "WSASYSCALLFAILURE";
				case 10108:	return "WSASERVICE_NOT_FOUND";
				case 10109:	return "WSATYPE_NOT_FOUND";
				case 10110:	return "WSA_E_NO_MORE";
				case 10111:	return "WSA_E_CANCELLED";
				case 10112:	return "WSAEREFUSED";
				case 11001:	return "WSAHOST_NOT_FOUND";
				case 11002:	return "WSATRY_AGAIN";
				case 11003:	return "WSANO_RECOVERY";
				case 11004:	return "WSANO_DATA";
				case 11005:	return "WSA_QOS_RECEIVERS";
				case 11006:	return "WSA_QOS_SENDERS";
				case 11007:	return "WSA_QOS_NO_SENDERS";
				case 11008:	return "WSA_QOS_NO_RECEIVERS";
				case 11009:	return "WSA_QOS_REQUEST_CONFIRMED";
				case 11010:	return "WSA_QOS_ADMISSION_FAILURE";
				case 11011:	return "WSA_QOS_POLICY_FAILURE";
				case 11012:	return "WSA_QOS_BAD_STYLE";
				case 11013:	return "WSA_QOS_BAD_OBJECT";
				case 11014:	return "WSA_QOS_TRAFFIC_CTRL_ERROR";
				case 11015:	return "WSA_QOS_GENERIC_ERROR";
				case 11016:	return "WSA_QOS_ESERVICETYPE";
				case 11017:	return "WSA_QOS_EFLOWSPEC";
				case 11018:	return "WSA_QOS_EPROVSPECBUF";
				case 11019:	return "WSA_QOS_EFILTERSTYLE";
				case 11020:	return "WSA_QOS_EFILTERTYPE";
				case 11021:	return "WSA_QOS_EFILTERCOUNT";
				case 11022:	return "WSA_QOS_EOBJLENGTH";
				case 11023:	return "WSA_QOS_EFLOWCOUNT";
				case 11024:	return "WSA_QOS_EUNKOWNPSOBJ";
				case 11025:	return "WSA_QOS_EPOLICYOBJ";
				case 11026:	return "WSA_QOS_EFLOWDESC";
				case 11027:	return "WSA_QOS_EPSFLOWSPEC";
				case 11028:	return "WSA_QOS_EPSFILTERSPEC";
				case 11029:	return "WSA_QOS_ESDMODEOBJ";
				case 11030:	return "WSA_QOS_ESHAPERATEOBJ";
				case 11031:	return "WSA_QOS_RESERVED_PETYPE";
				default:	return String.Format( "Unknown ({0})", errorCode );
			}
		}
		
		public static string ShortDescription( int errorCode )
		{
			switch( errorCode )
			{
				case 6:		return "Specified event object handle is invalid.";
				case 8:		return "Insufficient memory available.";
				case 87:	return "One or more parameters are invalid.";
				case 995:	return "Overlapped operation aborted.";
				case 996:	return "Overlapped I/O event object not in signaled state.";
				case 997:	return "Overlapped operations will complete later.";
				case 10004:	return "Interrupted function call.";
				case 10009:	return "File handle is not valid.";
				case 10013:	return "Permission denied.";
				case 10014:	return "Bad address.";
				case 10022:	return "Invalid argument.";
				case 10024:	return "Too many open files.";
				case 10035:	return "Resource temporarily unavailable.";
				case 10036:	return "Operation now in progress.";
				case 10037:	return "Operation already in progress.";
				case 10038:	return "Socket operation on nonsocket.";
				case 10039:	return "Destination address required.";
				case 10040:	return "Message too long.";
				case 10041:	return "Protocol wrong type for socket.";
				case 10042:	return "Bad protocol option.";
				case 10043:	return "Protocol not supported.";
				case 10044:	return "Socket type not supported.";
				case 10045:	return "Operation not supported.";
				case 10046:	return "Protocol family not supported.";
				case 10047:	return "Address family not supported by protocol family.";
				case 10048:	return "Address already in use.";
				case 10049:	return "Cannot assign requested address.";
				case 10050:	return "Network is down.";
				case 10051:	return "Network is unreachable.";
				case 10052:	return "Network dropped connection on reset.";
				case 10053:	return "Software caused connection abort.";
				case 10054:	return "Connection reset by peer.";
				case 10055:	return "No buffer space available.";
				case 10056:	return "Socket is already connected.";
				case 10057:	return "Socket is not connected.";
				case 10058:	return "Cannot send after socket shutdown.";
				case 10059:	return "Too many references.";
				case 10060:	return "Connection timed out.";
				case 10061:	return "Connection refused.";
				case 10062:	return "Cannot translate name.";
				case 10063:	return "Name too long.";
				case 10064:	return "Host is down.";
				case 10065:	return "No route to host.";
				case 10066:	return "Directory not empty.";
				case 10067:	return "Too many processes.";
				case 10068:	return "User quota exceeded.";
				case 10069:	return "Disk quota exceeded.";
				case 10070:	return "Stale file handle reference.";
				case 10071:	return "Item is remote.";
				case 10091:	return "Network subsystem is unavailable.";
				case 10092:	return "Winsock.dll version out of range.";
				case 10093:	return "Successful WSAStartup not yet performed.";
				case 10101:	return "Graceful shutdown in progress.";
				case 10102:	return "No more results.";
				case 10103:	return "Call has been canceled.";
				case 10104:	return "Procedure call table is invalid.";
				case 10105:	return "Service provider is invalid.";
				case 10106:	return "Service provider failed to initialize.";
				case 10107:	return "System call failure.";
				case 10108:	return "Service not found.";
				case 10109:	return "Class type not found.";
				case 10110:	return "No more results.";
				case 10111:	return "Call was canceled.";
				case 10112:	return "Database query was refused.";
				case 11001:	return "Host not found.";
				case 11002:	return "Nonauthoritative host not found.";
				case 11003:	return "This is a nonrecoverable error.";
				case 11004:	return "Valid name, no data record of requested type.";
				case 11005:	return "QOS receivers.";
				case 11006:	return "QOS senders.";
				case 11007:	return "No QOS senders.";
				case 11008:	return "QOS no receivers.";
				case 11009:	return "QOS request confirmed.";
				case 11010:	return "QOS admission error.";
				case 11011:	return "QOS policy failure.";
				case 11012:	return "QOS bad style.";
				case 11013:	return "QOS bad object.";
				case 11014:	return "QOS traffic control error.";
				case 11015:	return "QOS generic error.";
				case 11016:	return "QOS service type error.";
				case 11017:	return "QOS flowspec error.";
				case 11018:	return "Invalid QOS provider buffer.";
				case 11019:	return "Invalid QOS filter style.";
				case 11020:	return "Invalid QOS filter type.";
				case 11021:	return "Incorrect QOS filter count.";
				case 11022:	return "Invalid QOS object length.";
				case 11023:	return "Incorrect QOS flow count.";
				case 11024:	return "Unrecognized QOS object.";
				case 11025:	return "Invalid QOS policy object.";
				case 11026:	return "Invalid QOS flow descriptor.";
				case 11027:	return "Invalid QOS provider-specific flowspec.";
				case 11028:	return "Invalid QOS provider-specific filterspec.";
				case 11029:	return "Invalid QOS shape discard mode object.";
				case 11030:	return "Invalid QOS shaping rate object.";
				case 11031:	return "Reserved policy QOS element type.";
				default:	return String.Format( "Unknown ({0})", errorCode );
			}
		}

		public static string LongDescription( int errorCode )
		{
			switch( errorCode )
			{
				case 6:		return "An application attempts to use an event object, but the specified handle is not valid. Note that this error is returned by the operating system, so the error number may change in future releases of Windows.";
				case 8:		return "An application used a Windows Sockets function that directly maps to a Windows function. The Windows function is indicating a lack of required memory resources. Note that this error is returned by the operating system, so the error number may change in future releases of Windows.";
				case 87:	return "An application used a Windows Sockets function which directly maps to a Windows function. The Windows function is indicating a problem with one or more parameters. Note that this error is returned by the operating system, so the error number may change in future releases of Windows.";
				case 995:	return "An overlapped operation was canceled due to the closure of the socket, or the execution of the SIO_FLUSH command in WSAIoctl. Note that this error is returned by the operating system, so the error number may change in future releases of Windows.";
				case 996:	return "The application has tried to determine the status of an overlapped operation which is not yet completed. Applications that use WSAGetOverlappedResult (with the fWait flag set to FALSE) in a polling mode to determine when an overlapped operation has completed, get this error code until the operation is complete. Note that this error is returned by the operating system, so the error number may change in future releases of Windows.";
				case 997:	return "The application has initiated an overlapped operation that cannot be completed immediately. A completion indication will be given later when the operation has been completed. Note that this error is returned by the operating system, so the error number may change in future releases of Windows.";
				case 10004:	return "A blocking operation was interrupted by a call to WSACancelBlockingCall.";
				case 10009:	return "The file handle supplied is not valid.";
				case 10013:	return "An attempt was made to access a socket in a way forbidden by its access permissions. An example is using a broadcast address for sendto without broadcast permission being set using setsockopt(SO_BROADCAST).  Another possible reason for the WSAEACCES error is that when the bind function is called (on Windows NT 4.0 with SP4 and later), another application, service, or kernel mode driver is bound to the same address with exclusive access. Such exclusive access is a new feature of Windows NT 4.0 with SP4 and later, and is implemented by using the SO_EXCLUSIVEADDRUSE option.";
				case 10014:	return "The system detected an invalid pointer address in attempting to use a pointer argument of a call. This error occurs if an application passes an invalid pointer value, or if the length of the buffer is too small. For instance, if the length of an argument, which is a sockaddr structure, is smaller than the sizeof(sockaddr).";
				case 10022:	return "Some invalid argument was supplied (for example, specifying an invalid level to the setsockopt function). In some instances, it also refers to the current state of the socket—for instance, calling accept on a socket that is not listening.";
				case 10024:	return "Too many open sockets. Each implementation may have a maximum number of socket handles available, either globally, per process, or per thread.";
				case 10035:	return "This error is returned from operations on nonblocking sockets that cannot be completed immediately, for example recv when no data is queued to be read from the socket. It is a nonfatal error, and the operation should be retried later. It is normal for WSAEWOULDBLOCK to be reported as the result from calling connect on a nonblocking SOCK_STREAM socket, since some time must elapse for the connection to be established.";
				case 10036:	return "A blocking operation is currently executing. Windows Sockets only allows a single blocking operation—per- task or thread—to be outstanding, and if any other function call is made (whether or not it references that or any other socket) the function fails with the WSAEINPROGRESS error.";
				case 10037:	return "An operation was attempted on a nonblocking socket with an operation already in progress—that is, calling connect a second time on a nonblocking socket that is already connecting, or canceling an asynchronous request (WSAAsyncGetXbyY) that has already been canceled or completed.";
				case 10038:	return "An operation was attempted on something that is not a socket. Either the socket handle parameter did not reference a valid socket, or for select, a member of an fd_set was not valid.";
				case 10039:	return "A required address was omitted from an operation on a socket. For example, this error is returned if sendto is called with the remote address of ADDR_ANY.";
				case 10040:	return "A message sent on a datagram socket was larger than the internal message buffer or some other network limit, or the buffer used to receive a datagram was smaller than the datagram itself.";
				case 10041:	return "A protocol was specified in the socket function call that does not support the semantics of the socket type requested. For example, the ARPA Internet UDP protocol cannot be specified with a socket type of SOCK_STREAM.";
				case 10042:	return "An unknown, invalid or unsupported option or level was specified in a getsockopt or setsockopt call.";
				case 10043:	return "The requested protocol has not been configured into the system, or no implementation for it exists. For example, a socket call requests a SOCK_DGRAM socket, but specifies a stream protocol.";
				case 10044:	return "The support for the specified socket type does not exist in this address family. For example, the optional type SOCK_RAW might be selected in a socket call, and the implementation does not support SOCK_RAW sockets at all.";
				case 10045:	return "The attempted operation is not supported for the type of object referenced. Usually this occurs when a socket descriptor to a socket that cannot support this operation is trying to accept a connection on a datagram socket.";
				case 10046:	return "The protocol family has not been configured into the system or no implementation for it exists. This message has a slightly different meaning from WSAEAFNOSUPPORT. However, it is interchangeable in most cases, and all Windows Sockets functions that return one of these messages also specify WSAEAFNOSUPPORT.";
				case 10047:	return "An address incompatible with the requested protocol was used. All sockets are created with an associated address family (that is, AF_INET for Internet Protocols) and a generic protocol type (that is, SOCK_STREAM). This error is returned if an incorrect protocol is explicitly requested in the socket call, or if an address of the wrong family is used for a socket, for example, in sendto.";
				case 10048:	return "Typically, only one usage of each socket address (protocol/IP address/port) is permitted. This error occurs if an application attempts to bind a socket to an IP address/port that has already been used for an existing socket, or a socket that was not closed properly, or one that is still in the process of closing. For server applications that need to bind multiple sockets to the same port number, consider using setsockopt (SO_REUSEADDR). Client applications usually need not call bind at all—connect chooses an unused port automatically. When bind is called with a wildcard address (involving ADDR_ANY), a WSAEADDRINUSE error could be delayed until the specific address is committed. This could happen with a call to another function later, including connect, listen, WSAConnect, or WSAJoinLeaf.";
				case 10049:	return "The requested address is not valid in its context. This normally results from an attempt to bind to an address that is not valid for the local computer. This can also result from connect, sendto, WSAConnect, WSAJoinLeaf, or WSASendTo when the remote address or port is not valid for a remote computer (for example, address or port 0).";
				case 10050:	return "A socket operation encountered a dead network. This could indicate a serious failure of the network system (that is, the protocol stack that the Windows Sockets DLL runs over), the network interface, or the local network itself.";
				case 10051:	return "A socket operation was attempted to an unreachable network. This usually means the local software knows no route to reach the remote host.";
				case 10052:	return "The connection has been broken due to keep-alive activity detecting a failure while the operation was in progress. It can also be returned by setsockopt if an attempt is made to set SO_KEEPALIVE on a connection that has already failed.";
				case 10053:	return "An established connection was aborted by the software in your host computer, possibly due to a data transmission time-out or protocol error.";
				case 10054:	return "An existing connection was forcibly closed by the remote host. This normally results if the peer application on the remote host is suddenly stopped, the host is rebooted, the host or remote network interface is disabled, or the remote host uses a hard close (see setsockopt for more information on the SO_LINGER option on the remote socket). This error may also result if a connection was broken due to keep-alive activity detecting a failure while one or more operations are in progress. Operations that were in progress fail with WSAENETRESET. Subsequent operations fail with WSAECONNRESET.";
				case 10055:	return "An operation on a socket could not be performed because the system lacked sufficient buffer space or because a queue was full.";
				case 10056:	return "A connect request was made on an already-connected socket. Some implementations also return this error if sendto is called on a connected SOCK_DGRAM socket (for SOCK_STREAM sockets, the to parameter in sendto is ignored) although other implementations treat this as a legal occurrence.";
				case 10057:	return "A request to send or receive data was disallowed because the socket is not connected and (when sending on a datagram socket using sendto) no address was supplied. Any other type of operation might also return this error—for example, setsockopt setting SO_KEEPALIVE if the connection has been reset.";
				case 10058:	return "A request to send or receive data was disallowed because the socket had already been shut down in that direction with a previous shutdown call. By calling shutdown a partial close of a socket is requested, which is a signal that sending or receiving, or both have been discontinued.";
				case 10059:	return "Too many references to some kernel object.";
				case 10060:	return "A connection attempt failed because the connected party did not properly respond after a period of time, or the established connection failed because the connected host has failed to respond.";
				case 10061:	return "No connection could be made because the target computer actively refused it. This usually results from trying to connect to a service that is inactive on the foreign host—that is, one with no server application running.";
				case 10062:	return "Cannot translate a name.";
				case 10063:	return "A name component or a name was too long.";
				case 10064:	return "A socket operation failed because the destination host is down. A socket operation encountered a dead host. Networking activity on the local host has not been initiated. These conditions are more likely to be indicated by the error WSAETIMEDOUT.";
				case 10065:	return "A socket operation was attempted to an unreachable host. See WSAENETUNREACH.";
				case 10066:	return "Cannot remove a directory that is not empty.";
				case 10067:	return "A Windows Sockets implementation may have a limit on the number of applications that can use it simultaneously. WSAStartup may fail with this error if the limit has been reached.";
				case 10068:	return "Ran out of user quota.";
				case 10069:	return "Ran out of disk quota.";
				case 10070:	return "The file handle reference is no longer available.";
				case 10071:	return "The item is not available locally.";
				case 10091:	return "This error is returned by WSAStartup if the Windows Sockets implementation cannot function at this time because the underlying system it uses to provide network services is currently unavailable. Users should check: * That the appropriate Windows Sockets DLL file is in the current path. * That they are not trying to use more than one Windows Sockets implementation simultaneously. If there is more than one Winsock DLL on your system, be sure the first one in the path is appropriate for the network subsystem currently loaded. * The Windows Sockets implementation documentation to be sure all necessary components are currently installed and configured correctly.";
				case 10092:	return "The current Windows Sockets implementation does not support the Windows Sockets specification version requested by the application. Check that no old Windows Sockets DLL files are being accessed.";
				case 10093:	return "Either the application has not called WSAStartup or WSAStartup failed. The application may be accessing a socket that the current active task does not own (that is, trying to share a socket between tasks), or WSACleanup has been called too many times.";
				case 10101:	return "Returned by WSARecv and WSARecvFrom to indicate that the remote party has initiated a graceful shutdown sequence.";
				case 10102:	return "No more results can be returned by the WSALookupServiceNext function.";
				case 10103:	return "A call to the WSALookupServiceEnd function was made while this call was still processing. The call has been canceled.";
				case 10104:	return "The service provider procedure call table is invalid. A service provider returned a bogus procedure table to Ws2_32.dll. This is usually caused by one or more of the function pointers being NULL.";
				case 10105:	return "The requested service provider is invalid. This error is returned by the WSCGetProviderInfo and WSCGetProviderInfo32 functions if the protocol entry specified could not be found. This error is also returned if the service provider returned a version number other than 2.0.";
				case 10106:	return "The requested service provider could not be loaded or initialized. This error is returned if either a service provider's DLL could not be loaded (LoadLibrary failed) or the provider's WSPStartup or NSPStartup function failed.";
				case 10107:	return "A system call that should never fail has failed. This is a generic error code, returned under various conditions.  Returned when a system call that should never fail does fail. For example, if a call to WaitForMultipleEvents fails or one of the registry functions fails trying to manipulate the protocol/namespace catalogs.  Returned when a provider does not return SUCCESS and does not provide an extended error code. Can indicate a service provider implementation error.";
				case 10108:	return "No such service is known. The service cannot be found in the specified name space.";
				case 10109:	return "The specified class was not found.";
				case 10110:	return "No more results can be returned by the WSALookupServiceNext function.";
				case 10111:	return "A call to the WSALookupServiceEnd function was made while this call was still processing. The call has been canceled.";
				case 10112:	return "A database query failed because it was actively refused.";
				case 11001:	return "No such host is known. The name is not an official host name or alias, or it cannot be found in the database(s) being queried. This error may also be returned for protocol and service queries, and means that the specified name could not be found in the relevant database.";
				case 11002:	return "This is usually a temporary error during host name resolution and means that the local server did not receive a response from an authoritative server. A retry at some time later may be successful.";
				case 11003:	return "This indicates that some sort of nonrecoverable error occurred during a database lookup. This may be because the database files (for example, BSD-compatible HOSTS, SERVICES, or PROTOCOLS files) could not be found, or a DNS request was returned by the server with a severe error.";
				case 11004:	return "The requested name is valid and was found in the database, but it does not have the correct associated data being resolved for. The usual example for this is a host name-to-address translation attempt (using gethostbyname or WSAAsyncGetHostByName) which uses the DNS (Domain Name Server). An MX record is returned but no A record—indicating the host itself exists, but is not directly reachable.";
				case 11005:	return "At least one QOS reserve has arrived.";
				case 11006:	return "At least one QOS send path has arrived.";
				case 11007:	return "There are no QOS senders.";
				case 11008:	return "There are no QOS receivers.";
				case 11009:	return "The QOS reserve request has been confirmed.";
				case 11010:	return "A QOS error occurred due to lack of resources.";
				case 11011:	return "The QOS request was rejected because the policy system couldn't allocate the requested resource within the existing policy.";
				case 11012:	return "An unknown or conflicting QOS style was encountered.";
				case 11013:	return "A problem was encountered with some part of the filterspec or the provider-specific buffer in general.";
				case 11014:	return "An error with the underlying traffic control (TC) API as the generic QOS request was converted for local enforcement by the TC API. This could be due to an out of memory error or to an internal QOS provider error.";
				case 11015:	return "A general QOS error.";
				case 11016:	return "An invalid or unrecognized service type was found in the QOS flowspec.";
				case 11017:	return "An invalid or inconsistent flowspec was found in the QOS structure.";
				case 11018:	return "An invalid QOS provider-specific buffer.";
				case 11019:	return "An invalid QOS filter style was used.";
				case 11020:	return "An invalid QOS filter type was used.";
				case 11021:	return "An incorrect number of QOS FILTERSPECs were specified in the FLOWDESCRIPTOR.";
				case 11022:	return "An object with an invalid ObjectLength field was specified in the QOS provider-specific buffer.";
				case 11023:	return "An incorrect number of flow descriptors was specified in the QOS structure.";
				case 11024:	return "An unrecognized object was found in the QOS provider-specific buffer.";
				case 11025:	return "An invalid policy object was found in the QOS provider-specific buffer.";
				case 11026:	return "An invalid QOS flow descriptor was found in the flow descriptor list.";
				case 11027:	return "An invalid or inconsistent flowspec was found in the QOS provider-specific buffer.";
				case 11028:	return "An invalid FILTERSPEC was found in the QOS provider-specific buffer.";
				case 11029:	return "An invalid shape discard mode object was found in the QOS provider-specific buffer.";
				case 11030:	return "An invalid shaping rate object was found in the QOS provider-specific buffer.";
				case 11031:	return "A reserved policy element was found in the QOS provider-specific buffer.";
				default:	return String.Format( "Unknown ({0})", errorCode );
			}
		}
	}
}