﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace IntelliSite.CommonUtil
{
	public class StringPair : Tuple<string, string>
	{
		public StringPair( string item1, string item2 )
			: base( item1, item2 )
		{
		}

		public override int GetHashCode()
		{
#if STRING_PAIR_COMPARER_TUPLE_HASH
			return base.GetHashCode();
#else
			// http://stackoverflow.com/a/263416/4081

			const int prime1 = 31;
			const int prime2 = 486187739;

			unchecked
			{
				int hash = prime1;
				hash = hash * prime2 + Item1.GetHashCode();
				hash = hash * prime2 + Item2.GetHashCode();

				return hash;
			}
#endif
		}
	}

	public class StringPairComparer : EqualityComparer<Tuple<string, string>>
	{
		public StringPairComparer( StringComparison comparisonType )
		{
			this.ComparisonType = comparisonType;
		}

		public StringComparison ComparisonType { get; protected set; }

		public override bool Equals( Tuple<string, string> x, Tuple<string, string> y )
		{
			if( x == null || y == null )
				return false;

			return x.Item1.Equals( y.Item1, this.ComparisonType ) && x.Item2.Equals( y.Item2, this.ComparisonType );
		}

		public override int GetHashCode( Tuple<string, string> obj )
		{
#if STRING_PAIR_COMPARER_TUPLE_HASH
			return obj.GetHashCode();
#else
			// http://stackoverflow.com/a/263416/4081

			const int prime1 = 31;
			const int prime2 = 486187739;

			unchecked
			{
				int hash = prime1;
				hash = hash * prime2 + obj.Item1.GetHashCode();
				hash = hash * prime2 + obj.Item2.GetHashCode();

				return hash;
			}
#endif
		}

		private static readonly StringPairComparer _invariantComparer = 
			new StringPairComparer( StringComparison.InvariantCulture );

		public static StringPairComparer InvariantCulture { get { return _invariantComparer; } }
	}
}
