﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Threading.Tasks;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Json;
using IntelliSite.VideoCommon;

namespace StandaloneVideoView
{
	[DataContract]
	public class AvailableCamera
	{
		[DataMember]
		public string Name { get; set; }
		[DataMember]
		public bool IsCamera { get; set; }
		[DataMember]
		public bool IsPreset { get; set; }
		[DataMember]
		public bool PlaybackEnabled { get; set; }
		[DataMember]
		public bool PTZEnabled { get; set; }
		public Dictionary<string,string> PropertyDictionary { get; private set; }
		[DataMember]
		public List<EditProperty> Properties
		{
			get
			{
				List<EditProperty> properties = new List<EditProperty>();
				foreach( KeyValuePair<string,string> kvp in PropertyDictionary )
					properties.Add( new EditProperty( kvp.Key, kvp.Value ) );
				return properties;
			}
			set
			{
				if( PropertyDictionary != null )
					PropertyDictionary.Clear();
				else
					PropertyDictionary = new Dictionary<string, string>();

				foreach( EditProperty editProperty in value )
					PropertyDictionary[editProperty.Name] = editProperty.Value;
			}
		}
		[DataMember]
		public List<object> Children { get; private set; }

		public bool Connected( ISCameraBase driver ) { return driver != null && driver.IsConnected; }
		public bool Recording( ISCameraBase driver ) { return driver != null && driver.IsRecording; }

		public AvailableCamera()
		{
			Children = new List<object>();
			PropertyDictionary = new Dictionary<string, string>();
		}

		public AvailableCamera( string name, bool isCamera = false, bool playbackEnabled = true, bool ptzEnabled = true )
		{
			Name = name;
			IsCamera = isCamera;
			PlaybackEnabled = playbackEnabled;
			PTZEnabled = ptzEnabled;
			Children = new List<object>();
			PropertyDictionary = new Dictionary<string,string>();

			if( !isCamera )
			{
				PlaybackEnabled = false;
				PTZEnabled = false;
			}
		}

		public AvailableCamera AddAvailableCamera( string name, bool isCamera = false )
		{
			AvailableCamera availableCamera = new AvailableCamera( name, isCamera );
			Children.Add( availableCamera );
			return availableCamera;
		}

		public void RemoveAvailableCamera( AvailableCamera availableCamera )
		{
			Children.Remove( availableCamera );
		}

		public override string ToString()
		{
			string retVal = Name;
			if( IsCamera )
			{
				retVal += " IsCamera";
				if( PTZEnabled )
					retVal += " PTZ";
				if( PlaybackEnabled )
					retVal += " Playback";
			}
			return retVal;
		}
	}
}
