﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Threading.Tasks;
using System.Windows.Forms;

using IntelliSite.CommonUtil;
using IntelliSite.VideoCommon;

using VisioForge.Controls.UI.WinForms;
using VisioForge.Shared.IPCameraDB;
using VisioForge.Tools;
using VisioForge.Types;
using VisioForge.Types.OutputFormat;
using VisioForge.Types.Sources;

//using IntelliSite.DigitalWatchdog.Data;

using Newtonsoft.Json;
using System.Collections.Specialized;
using System.Diagnostics;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Web;

namespace IntelliSite.DigitalWatchdog.Driver
{
    public enum ProtocolType : byte
    {
        Http,
        Rtsp
    }

    public class DigitalWatchdogCamera : ISCameraBase
    {


        #region Private member variables

        private static PtzParameters _ptzParameters = new PtzParameters();

        private string _currentAddress = "";
        private string _currentPort = "";
        private ProtocolType _currentProtocol = ProtocolType.Rtsp;
        private string _currentUsername = "";
        private string _currentPassword = "";
        private string _currentCameraID = "";

        private VisioForge.Controls.UI.WinForms.VideoCapture _videoControl;

        private bool _errorEncountered = false;

        #endregion


        #region Constructors

        public DigitalWatchdogCamera(LoggerBase logger = null) : base(logger)
        {
            Logger.Write("Call DigitalWatchdog.Driver.DigitalWatchdogCamera.DigitalWatchdogCamera(logger:" + (logger == null ? "null" : logger.ToString()) + ")");

            _videoControl = null;
            VideoControlInit();
        }

        #endregion


        #region Properties

        public override bool IsPTZSupported
        {
            get
            {
                return true;
            }
        }

        public override System.Windows.Forms.Control VideoViewerWinForms
        {
            get
            {
                //return (System.Windows.Forms.Control)_videoControl;
                return (System.Windows.Forms.Control)_videoControl;
            }
            set
            {
                //_videoControl = (VlcControl)value;
                _videoControl = (VisioForge.Controls.UI.WinForms.VideoCapture)value;
            }
        }

        #endregion


        #region Event handlers

        private void CameraSetupReceived(object sender, VideoStatusArgs e)
        {

            try { 

                Logger.Write("Call DigitalWatchdog.Driver.DigitalWatchdogCamera.CameraSetupReceived(sender:" + sender.ToString() + ", e.Text:" + e.Text + ")");

                OnVideoSetupInfo(this, e);

            }
            catch (Exception ex)
            {
                Logger.Write("ERROR: " + ex.ToString());
                throw;
            }

        }
        
        private void CameraStateChanged(object sender, CameraStateEventArgs e)
        {

            try {

                Logger.Write("Call DigitalWatchdog.Driver.DigitalWatchdogCamera.CameraStateChanged(sender:" + sender.ToString() + ", e.State:" + e.State.ToString() + ")");

                string status = e.State == CameraState.StreamConnecting ? "Stream Connecting" : e.State.ToString();
                OnVideoStatus(sender, new VideoStatusArgs(status));
                if (e.State == CameraState.Streaming)
                {
                    OnStreaming(this, new EventArgs());
                }
            }
            catch (Exception ex)
            {
                Logger.Write("ERROR: " + ex.ToString());
                throw;
            }

        }
        /*
        public override void ChangeCamera(object cameraInfo)
        {

            try { 

                Logger.Write("Call DigitalWatchdog.Driver.DigitalWatchdogCamera.ChangeCamera(cameraInfo:" + cameraInfo.ToString() + " )");

                base.ChangeCamera(cameraInfo);

            }
            catch (Exception ex)
            {
                Logger.Write("ERROR: " + ex.ToString());
                throw;
            }

        }
        */
        public override void ChangeLocation(Point point)
        {

            try { 

                Logger.Write("Call DigitalWatchdog.Driver.DigitalWatchdogCamera.ChangeLocation(" + (point == null ? "point:null" : "point.X:" + point.X + ",point.Y:" + point.Y) + ")");

                base.ChangeLocation(point);
                _videoControl.Location = point;

            }
            catch (Exception ex)
            {
                Logger.Write("ERROR: " + ex.ToString());
                throw;
            }

        }

        public override void ChangeSize(Size size)
        {

            try { 

                Logger.Write("Call DigitalWatchdog.Driver.DigitalWatchdogCamera.ChangeSize(" + (size == null ? "size:null" : "size.Height:" + size.Height + ",size.Width:" + size.Width) + ")");

                base.ChangeSize(size);
                _videoControl.Size = size;

            }
            catch (Exception ex)
            {
                Logger.Write("ERROR: " + ex.ToString());
                throw;
            }

        }
                
        private void VideoControl_MouseHover(object sender, EventArgs e)
        {
            Logger.Write("Call DigitalWatchdog.Driver.DigitalWatchdogCamera.VideoControl_MouseHover(sender:" + sender.ToString() + ", e:" + e.ToString() + ")");

            // The driver must pass this action to the control so that the toolbar will popup when hovering over the window.
            if (this.Parent != null)
            {
                ((IntelliSite.WinForms.VideoView.VideoControl)this.Parent).OnMouseHover(sender, e);
            }
        }

        private void VideoControl_OnError(object sender, VisioForge.Types.ErrorsEventArgs e)
        {
            if (Logger != null)
            {
                switch (e.Level)
                {
                    case DebugLevel.Debug:
                        Logger.Write("Debug: " + e.Message);
                        break;
                    case DebugLevel.Error:
                        Logger.Write("Error: " + e.Message);
                        break;
                    case DebugLevel.Warning:
                        Logger.Write("Warning: " + e.Message);
                        break;
                };
            }
        }

        private void VideoControl_OnLicenseRequired(object sender, VisioForge.Types.LicenseEventArgs e)
        {
            if (Logger != null)
            {
                Logger.Write("VisioForge License required: " + e.Message);
                Logger.Write("VisioForge SDK state: " + _videoControl.SDK_State);
                Logger.Write("VisioForge SDK version: " + _videoControl.SDK_Version);
            };
        }
        #endregion


        #region Setup and initialization methods
        protected void VideoControlInit()
        {
            Logger.Write("Call DigitalWatchdog.Driver.DigitalWatchdogCamera.VideoControlInit()");
            if (_videoControl == null)
            {
                _videoControl = new VideoCapture();
                this._videoControl.Additional_Audio_CaptureDevice_MixChannels = false;
                this._videoControl.Audio_CaptureDevice = "";
                this._videoControl.Audio_CaptureDevice_CustomLatency = 0;
                this._videoControl.Audio_CaptureDevice_Format = "";
                this._videoControl.Audio_CaptureDevice_Format_UseBest = true;
                this._videoControl.Audio_CaptureDevice_Line = "";
                this._videoControl.Audio_CaptureDevice_MasterDevice = null;
                this._videoControl.Audio_CaptureDevice_MasterDevice_Format = null;
                this._videoControl.Audio_CaptureDevice_Path = null;
                this._videoControl.Audio_CaptureSourceFilter = null;
                this._videoControl.Audio_Channel_Mapper = null;
                this._videoControl.Audio_Decoder = null;
                this._videoControl.Audio_Effects_Enabled = false;
                this._videoControl.Audio_Effects_UseLegacyEffects = false;
                this._videoControl.Audio_Enhancer_Enabled = false;
                this._videoControl.Audio_OutputDevice = "Default DirectSound Device";
                this._videoControl.Audio_PCM_Converter = null;
                this._videoControl.Audio_PlayAudio = true;
                this._videoControl.Audio_RecordAudio = true;
                this._videoControl.Audio_Sample_Grabber_Enabled = false;
                this._videoControl.Audio_VUMeter_Enabled = false;
                this._videoControl.Audio_VUMeter_Pro_Enabled = false;
                this._videoControl.Audio_VUMeter_Pro_Volume = 100;
                this._videoControl.BackColor = System.Drawing.Color.Black;
                this._videoControl.Barcode_Reader_Enabled = false;
                this._videoControl.Barcode_Reader_Type = VisioForge.Types.VFBarcodeType.Auto;
                this._videoControl.BDA_Source = null;
                this._videoControl.ChromaKey = null;
                this._videoControl.Custom_Source = null;
                this._videoControl.Debug_Dir = "";
                this._videoControl.Debug_Mode = false;
                this._videoControl.Decklink_Input = VisioForge.Types.DecklinkInput.Auto;
                this._videoControl.Decklink_Input_Capture_Timecode_Source = VisioForge.Types.DecklinkCaptureTimecodeSource.Auto;
                this._videoControl.Decklink_Input_IREUSA = false;
                this._videoControl.Decklink_Input_SMPTE = false;
                this._videoControl.Decklink_Output = null;
                this._videoControl.Decklink_Source = null;
                this._videoControl.DirectCapture_Muxer = null;
                this._videoControl.DV_Decoder_Video_Resolution = VisioForge.Types.VFDVVideoResolution.Full;
                this._videoControl.Face_Tracking = null;
                this._videoControl.IP_Camera_Source = null;
                this._videoControl.Mode = VisioForge.Types.VFVideoCaptureMode.VideoCapture;
                this._videoControl.Motion_Detection = null;
                this._videoControl.Motion_DetectionEx = null;
                this._videoControl.MPEG_Audio_Decoder = "";
                this._videoControl.MPEG_Demuxer = null;
                this._videoControl.MPEG_Video_Decoder = "";
                this._videoControl.MultiScreen_Enabled = false;
                this._videoControl.Name = "_videoControl";
                this._videoControl.Network_Streaming_Audio_Enabled = false;
                this._videoControl.Network_Streaming_Enabled = true;
                this._videoControl.Network_Streaming_Format = VisioForge.Types.VFNetworkStreamingFormat.WMV;
                this._videoControl.Network_Streaming_Network_Port = 10;
                this._videoControl.Network_Streaming_Output = null;
                this._videoControl.Network_Streaming_URL = "";
                this._videoControl.Network_Streaming_WMV_Maximum_Clients = 10;
                this._videoControl.Output_Filename = "";
                this._videoControl.Output_Format = null;
                this._videoControl.PIP_AddSampleGrabbers = false;
                this._videoControl.PIP_ChromaKeySettings = null;
                this._videoControl.PIP_Mode = VisioForge.Types.VFPIPMode.Custom;
                this._videoControl.PIP_ResizeQuality = VisioForge.Types.VFPIPResizeQuality.RQ_NN;
                this._videoControl.Push_Source = null;
                this._videoControl.Screen_Capture_Source = null;
                this._videoControl.SeparateCapture_AutostartCapture = false;
                this._videoControl.SeparateCapture_Enabled = false;
                this._videoControl.SeparateCapture_Filename_Mask = "output %yyyy-%MM-%dd %hh-%mm-%ss.%ext";
                this._videoControl.SeparateCapture_FileSizeThreshold = ((long)(0));
                this._videoControl.SeparateCapture_GMFMode = true;
                this._videoControl.SeparateCapture_Mode = VisioForge.Types.VFSeparateCaptureMode.Normal;
                this._videoControl.SeparateCapture_TimeThreshold = ((long)(0));
                this._videoControl.Start_DelayEnabled = false;
                this._videoControl.Tags = null;
                this._videoControl.Timeshift_Settings = null;
                this._videoControl.TVTuner_Channel = 0;
                this._videoControl.TVTuner_Country = "";
                this._videoControl.TVTuner_FM_Tuning_StartFrequency = 80000000;
                this._videoControl.TVTuner_FM_Tuning_Step = 160000000;
                this._videoControl.TVTuner_FM_Tuning_StopFrequency = 0;
                this._videoControl.TVTuner_Frequency = 0;
                this._videoControl.TVTuner_InputType = "";
                this._videoControl.TVTuner_Mode = VisioForge.Types.VFTVTunerMode.Default;
                this._videoControl.TVTuner_Name = "";
                this._videoControl.TVTuner_TVFormat = VisioForge.Types.VFTVTunerVideoFormat.PAL_D;
                this._videoControl.Video_CaptureDevice = "";
                this._videoControl.Video_CaptureDevice_Format = "";
                this._videoControl.Video_CaptureDevice_Format_UseBest = true;
                this._videoControl.Video_CaptureDevice_FrameRate = 0D;
                this._videoControl.Video_CaptureDevice_InternalMPEGEncoder_Name = "";
                this._videoControl.Video_CaptureDevice_IsAudioSource = false;
                this._videoControl.Video_CaptureDevice_Path = null;
                this._videoControl.Video_CaptureDevice_UseClosedCaptions = false;
                this._videoControl.Video_CaptureDevice_UseRAWSampleGrabber = false;
                this._videoControl.Video_Crop = null;
                this._videoControl.Video_Decoder = null;
                this._videoControl.Video_Effects_AllowMultipleStreams = false;
                this._videoControl.Video_Effects_Enabled = false;

                VisioForge.Types.VideoRendererSettingsWinForms videoRendererSettingsWinForms3 = new VisioForge.Types.VideoRendererSettingsWinForms();
                videoRendererSettingsWinForms3.Aspect_Ratio_Override = false;
                videoRendererSettingsWinForms3.Aspect_Ratio_X = 0;
                videoRendererSettingsWinForms3.Aspect_Ratio_Y = 0;
                videoRendererSettingsWinForms3.BackgroundColor = System.Drawing.Color.Black;
                // TODO: Code generation for '' failed because of Exception 'Invalid Primitive Type: System.IntPtr. Consider using CodeObjectCreateExpression.'.
                videoRendererSettingsWinForms3.Deinterlace_EVR_Mode = VisioForge.Types.EVRDeinterlaceMode.Auto;
                videoRendererSettingsWinForms3.Deinterlace_VMR9_Mode = null;
                videoRendererSettingsWinForms3.Deinterlace_VMR9_UseDefault = true;
                videoRendererSettingsWinForms3.Flip_Horizontal = false;
                videoRendererSettingsWinForms3.Flip_Vertical = false;
                videoRendererSettingsWinForms3.RotationAngle = 0;
                videoRendererSettingsWinForms3.StretchMode = VisioForge.Types.VFVideoRendererStretchMode.Letterbox;
                videoRendererSettingsWinForms3.Video_Renderer = VisioForge.Types.VFVideoRenderer.VideoRenderer;
                videoRendererSettingsWinForms3.VideoRendererInternal = VisioForge.Types.VFVideoRendererInternal.VideoRenderer;
                videoRendererSettingsWinForms3.Zoom_Ratio = 0;
                videoRendererSettingsWinForms3.Zoom_ShiftX = 0;
                videoRendererSettingsWinForms3.Zoom_ShiftY = 0;
                this._videoControl.Video_Renderer = videoRendererSettingsWinForms3;

                this._videoControl.Video_Resize = null;
                this._videoControl.Video_ResizeOrCrop_Enabled = false;
                this._videoControl.Video_Rotation = VisioForge.Types.VFRotateMode.RotateNone;
                this._videoControl.Video_Sample_Grabber_Enabled = false;
                this._videoControl.Video_Sample_Grabber_UseForVideoEffects = false;
                this._videoControl.Video_Still_Frames_Grabber_Enabled = false;
                this._videoControl.Virtual_Camera_Output_Enabled = false;
                this._videoControl.Virtual_Camera_Output_LicenseKey = null;
                this._videoControl.VLC_Path = null;

                this._videoControl.SetLicenseKey("1DCD-8B1A-6863-E7FC-F7B4-9C6B", "OSSI,LLC", "dnovak@ossi-usa.com");

                this._videoControl.OnError += new System.EventHandler<VisioForge.Types.ErrorsEventArgs>(this.VideoControl_OnError);
                this._videoControl.OnLicenseRequired += new System.EventHandler<VisioForge.Types.LicenseEventArgs>(this.VideoControl_OnLicenseRequired);
            }
        }

        public override void Init()
        {
            try
            {
                Logger.Write("Call DigitalWatchdog.Driver.DigitalWatchdogCamera.Init()");

                base.Init();
            }
            catch(Exception ex)
            {
                Logger.Write("ERROR: " + ex.ToString());
                throw new Exception("Digital Watchdog driver failed to initialize.", ex);
            }

        }
    #endregion

    #region Connect/Disconnect methods
        private bool Do_Connect()
        {
            bool success = false;
            try
            {
                if (IsConnected) Disconnect();
                if (_videoControl == null) VideoControlInit();

                string cameraURL = null;
                if (Convert.ToBoolean(CameraProperty("DirectConnect")))
                {
                    cameraURL = CameraProperty("CustomURL");
                    if (string.IsNullOrWhiteSpace(cameraURL))
                    {
                        string selectedURL = CameraProperty("SelectedURL");
                        cameraURL = CameraProperty("DetectedURL" + selectedURL);
                    }

                    if (!cameraURL.Contains("@")) //Assume it already contains the username/password so no need to attempt to add.
                    {
                        if (!Convert.ToBoolean(CameraProperty("UseAnonymous")))
                        {
                            string username = CameraProperty("Username");
                            if (String.IsNullOrWhiteSpace(username)) username = _currentUsername;

                            string password = CameraProperty("Password");
                            if (String.IsNullOrWhiteSpace(password)) password = _currentPassword;

                            int afterdoubleslash = cameraURL.IndexOf("//") + 2;
                            cameraURL = cameraURL.Insert(afterdoubleslash, username + ":" + password + "@");
                        }
                    }
                }
                else
                {
                    if (_currentProtocol == ProtocolType.Rtsp)// String.Compare("rtsp", protocol, true) == 0)
                        cameraURL = "rtsp://" + _currentUsername + ":" + _currentPassword + "@" + _currentAddress + ":" + _currentPort + "/" + this._currentCameraID;
                    else if (_currentProtocol == ProtocolType.Http)
                    {
                        cameraURL = "http://" + _currentUsername + ":" + _currentPassword + "@" + _currentAddress + ":" + _currentPort + "/media/" + this._currentCameraID + ".webm";
                    }
                }

                if (!String.IsNullOrEmpty(cameraURL))
                {
                    Logger.Write("    cameraURL: " + cameraURL);
                    this._errorEncountered = false;

                    _videoControl.IP_Camera_Source = new IPCameraSourceSettings { URL = cameraURL };
                    _videoControl.IP_Camera_Source.Type = VFIPSource.Auto_LAV;
                    _videoControl.IP_Camera_Source.AudioCapture = true;

                    if (VideoCapture.Filter_Supported_EVR())
                    {
                        _videoControl.Video_Renderer.Video_Renderer = VFVideoRenderer.EVR;
                    }
                    else if (VideoCapture.Filter_Supported_VMR9())
                    {
                        _videoControl.Video_Renderer.Video_Renderer = VFVideoRenderer.VMR9;
                    }
                    else
                    {
                        _videoControl.Video_Renderer.Video_Renderer = VFVideoRenderer.VideoRenderer;
                    }

                    _videoControl.Video_Sample_Grabber_Enabled = true;

                    _videoControl.Mode = VFVideoCaptureMode.IPPreview;
                    
                    _videoControl.IP_Camera_Source.Login = String.Empty;
                    _videoControl.IP_Camera_Source.Password = String.Empty;

                    _videoControl.Debug_Mode = true;
                    _videoControl.Debug_Dir = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments) + "\\VisioForge\\";

                    this._videoControl.SetLicenseKey("1DCD-8B1A-6863-E7FC-F7B4-9C6B", "OSSI,LLC", "dnovak@ossi-usa.com");
                    _videoControl.Start();
                    OnStreaming(this, new EventArgs());

                    //_videoControl.Visible = true;
                    success = true;
                    InPlaybackMode = false;
                }          

                return success;
            }
            catch (Exception ex)
            {
                Logger.Write("ERROR: " + ex.ToString());
                throw new Exception("Digital Watchdog driver failed to connect.", ex);
            }
        }

        public override bool Connect(object parent)
        {
            try
            {
                Logger.Write("Call DigitalWatchdog.Driver.DigitalWatchdogCamera.Connect(parent:" + (parent == null ? "null" : parent.ToString()) + ")");

                Parent = parent;

                if (!String.IsNullOrEmpty(CameraName))
                {
                    OnVideoStatus(this, new VideoStatusArgs("Connecting to " + CameraName));
                    //_videoControl.Visible = true;

                    _currentAddress = CameraProperty("ServerIPAddress");
                    _currentPort = CameraProperty("ServerPort");
                    _currentUsername = CameraProperty("ServerUsername");
                    _currentPassword = CameraProperty("ServerPassword");
                    _currentCameraID = CameraProperty("CameraID");
                    _currentProtocol = ProtocolType.Rtsp;
                    Enum.TryParse<ProtocolType>(CameraProperty("ServerProtocol"), out _currentProtocol);

                    IsConnected = Do_Connect();
                }
                else
                    IsConnected = false;

                return IsConnected;
            }
            catch (Exception ex)
            {
                Logger.Write("ERROR: " + ex.ToString());
                throw new Exception("Digital Watchdog driver failed to connect.", ex);
            }
        }

        public override void Disconnect()
        {
            try
            {
                Logger.Write("Call DigitalWatchdog.Driver.DigitalWatchdogCamera.Disconnect()   Camera Name: " + CameraName);

                if (!IsConnected) return;

                _videoControl.Stop();
                

                this.IsConnected = false;
                this.InPlaybackMode = false;
                this.IsRecording = false;

                if (!this._errorEncountered)
                    OnVideoStatus(this, new VideoStatusArgs("Disconnected"));
                else
                {
                    this._errorEncountered = false;
                }

                base.Disconnect();
            }
            catch (Exception ex)
            {
                Logger.Write("ERROR: " + ex.ToString());
                //throw new Exception("Digital Watchdog driver failed to disconnect.", ex);
            }

        }

        #endregion


        #region Camera methods

 

        #endregion



    }

}
