﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using IntelliSite.VideoCommon;

namespace IntelliSite.WinForms.VideoView
{
	public partial class AlarmsForm : Form
	{
		public VideoControl VideoControl { get; set; }
		public List<VideoAlarm> VideoAlarms { get; set; }

		public AlarmsForm( VideoControl videoControl, List<VideoAlarm> videoAlarms )
		{
			InitializeComponent();

			alarmsGrid.AutoGenerateColumns = false;
			rangeTypeCombo.SelectedIndex = 0;
			startTimePicker.Value = DateTime.Now.AddHours( -1 );
			endTimePicker.Value = DateTime.Now;

			VideoControl = videoControl;
			VideoAlarms = videoAlarms;
			alarmsGrid.DataSource = videoAlarms;
		}

		private void OnRetrieve( object sender, EventArgs e )
		{
			VideoAlarms = VideoControl.GetAlarms( startTimePicker.Value, endTimePicker.Value, systemAlarmsCheckBox.Checked, bookmarksCheckBox.Checked, eventsCheckBox.Checked, alarmsCheckBox.Checked );
			alarmsGrid.DataSource = VideoAlarms;
		}

		#region private void OnRangeTypeChanged( object sender, EventArgs e )
		private void OnRangeTypeChanged( object sender, EventArgs e )
		{
			switch( rangeTypeCombo.SelectedIndex )
			{
				case 0:	// Last Hour
					startTimePicker.Value = DateTime.Now.AddHours( -1 );
					endTimePicker.Value = DateTime.Now;
					break;
				case 1:	// Yesterday
					startTimePicker.Value = DateTime.Now.Date.AddDays( -1 );
					endTimePicker.Value = startTimePicker.Value + new TimeSpan( 23, 59, 59 );
					break;
				case 2:	// Last Week
					startTimePicker.Value = DateTime.Now.Date.AddDays( ( (int)DateTime.Now.DayOfWeek + 7 ) * -1 );
					endTimePicker.Value = startTimePicker.Value + new TimeSpan( 6, 23, 59, 59 );
					break;
				case 3: // Last Month
					if( DateTime.Now.Month != 1 )
					{
						startTimePicker.Value = new DateTime( DateTime.Now.Year, DateTime.Now.Month - 1, 1, 0, 0, 0 );
						endTimePicker.Value = new DateTime( DateTime.Now.Year, DateTime.Now.Month, 1 ).AddSeconds( -1 );
					}
					else
					{
						startTimePicker.Value = new DateTime( DateTime.Now.Year-1, 12, 1, 0, 0, 0 );
						endTimePicker.Value = new DateTime( DateTime.Now.Year-1, 12, 31, 23, 59, 59 );
					}
					break;
				case 4: // Last Quarter
					if( DateTime.Now.Month > 3 )
					{
						int quarter = ( DateTime.Now.Month - 1 ) / 3 - 1;
						startTimePicker.Value = new DateTime( DateTime.Now.Year, quarter * 3 + 1, 1, 0, 0, 0 );
						endTimePicker.Value = new DateTime( DateTime.Now.Year, quarter * 3 + 4, 1 ).AddSeconds( -1 );
					}
					else
					{
						startTimePicker.Value = new DateTime( DateTime.Now.Year-1, 10, 1, 0, 0, 0 );
						endTimePicker.Value = new DateTime( DateTime.Now.Year-1, 12, 31, 23, 59, 59 );
					}
					break;
				case 5: // Last Year
					startTimePicker.Value = new DateTime( DateTime.Now.Year - 1, 1, 1, 0, 0, 0 );
					endTimePicker.Value = new DateTime( DateTime.Now.Year - 1, 12, 31, 23, 59, 59 );
					break;
			}

			startTimePicker.Enabled = rangeTypeCombo.SelectedIndex == 6;
			endTimePicker.Enabled = rangeTypeCombo.SelectedIndex == 6;
		}
		#endregion
		#region private void OnDoubleClick( object sender, DataGridViewCellEventArgs e )
		private void OnDoubleClick( object sender, DataGridViewCellEventArgs e )
		{
			if( VideoAlarms != null && e.RowIndex > 0 && e.RowIndex < VideoAlarms.Count )
			{
				VideoControl.PlaybackTime = VideoAlarms[e.RowIndex].StartTime;
				VideoControl.OnPlaybackTimeChanged();
				Close();
			}
		}
		#endregion
	}
}
