﻿using System;
using System.Windows.Forms;
using IntelliSite.VideoCommon;

namespace IntelliSite.WinForms.VideoView
{
	public partial class PTZForm : Form
	{
		public VideoControl VideoControl { get; set; }
		
		public PTZForm( VideoControl videoControl )
		{
			InitializeComponent();
			
			VideoControl = videoControl;
			VideoControl.PTZConnect();

			EnableControls( VideoControl.IsPTZConnected );
		}

		private void OnFormClosing( object sender, FormClosingEventArgs e )
		{
			VideoControl.PTZDisconnect();
		}

		public delegate void EnableControlsDelegate( bool enabled );
		public void EnableControls( bool enabled )
		{
			string text = "PTZ " + ( enabled ? "Ready" : "Initializing" );
			if( VideoControl.View.NameProperty != null )
				text += " - " + VideoControl.View.NameProperty.GetValue( VideoControl.CameraInfo, null );
			Text = text;

			BackgroundImage = enabled ? global::IntelliSite.WinForms.VideoView.Properties.Resources.ptz_pad : global::IntelliSite.WinForms.VideoView.Properties.Resources.ptz_pad_offline;

			BackColor = enabled ? System.Drawing.Color.Black : System.Drawing.Color.LightGray;
			stopButton.Enabled = enabled;
			zoomInButton.Enabled = enabled;
			zoomOutButton.Enabled = enabled;
			focusFarButton.Enabled = enabled;
			focusNearButton.Enabled = enabled;
			irisCloseButton.Enabled = enabled;
			irisOpenButton.Enabled = enabled;
			ptzSpeedDecreasebutton.Enabled = enabled;
			ptzSpeedIncreaseButton.Enabled = enabled;
			zoomSpeedDecreaseButton.Enabled = enabled;
			zoomSpeedIncreaseButton.Enabled = enabled;
			focusSpeedDecreaseButton.Enabled = enabled;
			focusSpeedIncreaseButton.Enabled = enabled;
			irisSpeedDecreaseButton.Enabled = enabled;
			irisSpeedIncreaseButton.Enabled = enabled;
			downRghtButton.Enabled = enabled;
			downButton.Enabled = enabled;
			downLeftButton.Enabled = enabled;
			rightButton.Enabled = enabled;
			leftButton.Enabled = enabled;
			upRightButton.Enabled = enabled;
			upButton.Enabled = enabled;
			upLeftbutton.Enabled = enabled;

			hidePresetLabel.Visible = true;
			presetCombo.Visible = false;
			presetCombo.Enabled = false;
		}

		private void OnPTZNW( object sender, MouseEventArgs e )				{ VideoControl.PTZ( PTZCommands.PTZUpLeft, true ); }
		private void OnPTZN( object sender, MouseEventArgs e )				{ VideoControl.PTZ( PTZCommands.PTZUp, true ); }
		private void OnPTZNE( object sender, MouseEventArgs e )				{ VideoControl.PTZ( PTZCommands.PTZUpRight, true ); }
		private void OnPTZW( object sender, MouseEventArgs e )				{ VideoControl.PTZ( PTZCommands.PTZLeft, true ); }
		private void OnPTZE( object sender, MouseEventArgs e )				{ VideoControl.PTZ( PTZCommands.PTZRight, true ); }
		private void OnPTZSW( object sender, MouseEventArgs e )				{ VideoControl.PTZ( PTZCommands.PTZDownLeft, true ); }
		private void OnPTZS( object sender, MouseEventArgs e )				{ VideoControl.PTZ( PTZCommands.PTZDown, true ); }
		private void OnPTZSE( object sender, MouseEventArgs e )				{ VideoControl.PTZ( PTZCommands.PTZDownRight, true ); }
		private void OnHome( object sender, MouseEventArgs e )				{ VideoControl.PTZ( PTZCommands.PTZHome, true ); }
		private void OnPTZSpeedUp( object sender, MouseEventArgs e )		{ VideoControl.PTZ( PTZCommands.PTZSpeedUp, true ); }
		private void OnPTZSpeedDown( object sender, MouseEventArgs e )		{ VideoControl.PTZ( PTZCommands.PTZSpeedDown, true ); }
		private void OnZoomIn( object sender, MouseEventArgs e )			{ VideoControl.PTZ( PTZCommands.ZoomIn, true ); }
		private void OnZoomOut( object sender, MouseEventArgs e )			{ VideoControl.PTZ( PTZCommands.ZoomOut, true ); }
		private void OnZoomSpeedUp( object sender, MouseEventArgs e )		{ VideoControl.PTZ( PTZCommands.ZoomSpeedUp, true ); }
		private void OnZoomSpeedDown( object sender, MouseEventArgs e )		{ VideoControl.PTZ( PTZCommands.ZoomSpeedDown, true ); }
		private void OnFocusNear( object sender, MouseEventArgs e )			{ VideoControl.PTZ( PTZCommands.FocusNear, true ); }
		private void OnFocusFar( object sender, MouseEventArgs e )			{ VideoControl.PTZ( PTZCommands.FocusFar, true ); }
		private void OnFocusSpeedUp( object sender, MouseEventArgs e )		{ VideoControl.PTZ( PTZCommands.FocusSpeedUp, true ); }
		private void OnFocusSpeedDown( object sender, MouseEventArgs e )	{ VideoControl.PTZ( PTZCommands.FocusSpeedDown, true ); }
		private void OnIrisOpen( object sender, MouseEventArgs e )			{ VideoControl.PTZ( PTZCommands.IrisOpen, true ); }
		private void OnIrisClose( object sender, MouseEventArgs e )			{ VideoControl.PTZ( PTZCommands.IrisClose, true ); }
		private void OnIrisSpeedUp( object sender, MouseEventArgs e )		{ VideoControl.PTZ( PTZCommands.IrisSpeedUp, true ); }
		private void OnIrisSpeedDown( object sender, MouseEventArgs e )		{ VideoControl.PTZ( PTZCommands.IrisSpeedDown, true ); }

		private void OnPTZNWUp( object sender, MouseEventArgs e )			{ if( e.Button == MouseButtons.Left ) VideoControl.PTZ( PTZCommands.PTZUpLeft, false ); }
		private void OnPTZNUp( object sender, MouseEventArgs e )			{ if( e.Button == MouseButtons.Left ) VideoControl.PTZ( PTZCommands.PTZUp, false ); }
		private void OnPTZNEUp( object sender, MouseEventArgs e )			{ if( e.Button == MouseButtons.Left ) VideoControl.PTZ( PTZCommands.PTZUpRight, false ); }
		private void OnPTZWUp( object sender, MouseEventArgs e )			{ if( e.Button == MouseButtons.Left ) VideoControl.PTZ( PTZCommands.PTZLeft, false ); }
		private void OnPTZEUp( object sender, MouseEventArgs e )			{ if( e.Button == MouseButtons.Left ) VideoControl.PTZ( PTZCommands.PTZRight, false ); }
		private void OnPTZSWUp( object sender, MouseEventArgs e )			{ if( e.Button == MouseButtons.Left ) VideoControl.PTZ( PTZCommands.PTZDownLeft, false ); }
		private void OnPTZSUp( object sender, MouseEventArgs e )			{ if( e.Button == MouseButtons.Left ) VideoControl.PTZ( PTZCommands.PTZDown, false ); }
		private void OnPTZSEUp( object sender, MouseEventArgs e )			{ if( e.Button == MouseButtons.Left ) VideoControl.PTZ( PTZCommands.PTZDownRight, false ); }
		private void OnStopUp( object sender, MouseEventArgs e )			{ if( e.Button == MouseButtons.Left ) VideoControl.PTZ( PTZCommands.PTZHome, false ); }
		private void OnPTZSpeedUpUp( object sender, MouseEventArgs e )		{ if( e.Button == MouseButtons.Left ) VideoControl.PTZ( PTZCommands.PTZSpeedUp, false ); }
		private void OnPTZSpeedDownUp( object sender, MouseEventArgs e )	{ if( e.Button == MouseButtons.Left ) VideoControl.PTZ( PTZCommands.PTZSpeedDown, false ); }
		private void OnZoomInUp( object sender, MouseEventArgs e )			{ if( e.Button == MouseButtons.Left ) VideoControl.PTZ( PTZCommands.ZoomIn, false ); }
		private void OnZoomOutUp( object sender, MouseEventArgs e )			{ if( e.Button == MouseButtons.Left ) VideoControl.PTZ( PTZCommands.ZoomOut, false ); }
		private void OnZoomSpeedUpUp( object sender, MouseEventArgs e )		{ if( e.Button == MouseButtons.Left ) VideoControl.PTZ( PTZCommands.ZoomSpeedUp, false ); }
		private void OnZoomSpeedDownUp( object sender, MouseEventArgs e )	{ if( e.Button == MouseButtons.Left ) VideoControl.PTZ( PTZCommands.ZoomSpeedDown, false ); }
		private void OnFocusNearUp( object sender, MouseEventArgs e )		{ if( e.Button == MouseButtons.Left ) VideoControl.PTZ( PTZCommands.FocusNear, false ); }
		private void OnFocusFarUp( object sender, MouseEventArgs e )		{ if( e.Button == MouseButtons.Left ) VideoControl.PTZ( PTZCommands.FocusFar, false ); }
		private void OnFocusSpeedUpUp( object sender, MouseEventArgs e )	{ if( e.Button == MouseButtons.Left ) VideoControl.PTZ( PTZCommands.FocusSpeedUp, false ); }
		private void OnFocusSpeedDownUp( object sender, MouseEventArgs e )	{ if( e.Button == MouseButtons.Left ) VideoControl.PTZ( PTZCommands.FocusSpeedDown, false ); }
		private void OnIrisOpenUp( object sender, MouseEventArgs e )		{ if( e.Button == MouseButtons.Left ) VideoControl.PTZ( PTZCommands.IrisOpen, false ); }
		private void OnIrisCloseUp( object sender, MouseEventArgs e )		{ if( e.Button == MouseButtons.Left ) VideoControl.PTZ( PTZCommands.IrisClose, false ); }
		private void OnIrisSpeedUpUp( object sender, MouseEventArgs e )		{ if( e.Button == MouseButtons.Left ) VideoControl.PTZ( PTZCommands.IrisSpeedUp, false ); }
		private void OnIrisSpeedDownUp( object sender, MouseEventArgs e )	{ if( e.Button == MouseButtons.Left ) VideoControl.PTZ( PTZCommands.IrisSpeedDown, false ); }
	}
}
